/*
	$Id: FileUpload.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.fileupload;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import java.util.Collections;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.net.SocketTimeoutException;

import javax.servlet.ServletRequest;
import javax.servlet.ServletInputStream;

import net.sf.basedb.util.FileUtil;

/**
	Objects of this class are used to handle file upload from browsers using
	the standard mechanism described in <a href="http://www.ietf.org/rfc/rfc1867.txt">RFC1867</a>.

	@author Nicklas
	@version 2.0
	@base.modified $Date$
*/
public class FileUpload
{

	/**
		Debug file upload.
	*/
	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.clients.web.fileupload.FileUpload");
	
	private boolean debugEnabled = log.isDebugEnabled();

	/**
		The default timeout to use when waiting for the next data packet to
		arrive.
	*/
	public static final long DEFAULT_TIMEOUT = 60000;
	
	private static final String EMPTY_STRING = "";
	
	/**
		Contains information about regular input field parameters
		name --> value
	*/
	private Map<String, List<String>> parameters = null;

	/**
		Contains information about uploaded files
		name --> UploadedFile
	*/
	private Map<String, List<UploadedFile>> files = null;

	/**
		The stream coming from the browser.
	*/
	private ServletInputStream in;
	
	/**
		The status of the upload.
	*/
	private UploadStatus status;
	
	/**
		Progress information for the upload
	*/
	private FileUploadProgress progress;
	
	/**
		The multipart/formdata boundary 
	*/
	private byte[] boundary;
	
	/**
		The maximum transefer rate (bytes/second)
	*/
	private int transferRate = 0;

	/**
		The timeout in milliseconds that we wait for more data to arrive.
		Default value is 60000 = 60 seconds.
	*/
	private long timeout = DEFAULT_TIMEOUT;
	
	/**
		Create a new <code>FileUpload</code> object which will read the posted
		data from the given <code>ServletRequest</code> object.

		@param request The <code>ServletRequest</code> object containg information
			about the request (and file upload)
		@throws IOException If there is an error reading from the request stream
		@throws UploadAbortedException If the upload was aborted
	*/
	public FileUpload(ServletRequest request)
		throws IOException, UploadAbortedException
	{
		this(request, 0, DEFAULT_TIMEOUT);
	}

	/**
		Create a new <code>FileUpload</code> object which will read the posted
		data from the given <code>ServletRequest</code> object limiting the
		transfer rate.

		@param request The <code>ServletRequest</code> object containg information
			about the request (and file upload)
		@param transferRate The maximum transfer rate in bytes per second, or 0
			to transfer as fast as possible
		@param timeout The timeout in milliseconds to wait for the next packet
			of data to arrive
		@throws IOException If there is an error reading from the request stream
		@throws UploadAbortedException If the upload was aborted
	*/
	public FileUpload(ServletRequest request, int transferRate, long timeout)
		throws IOException, UploadAbortedException
	{
		this.parameters = new HashMap<String, List<String>>();
		this.files = new HashMap<String, List<UploadedFile>>();
		this.status = new UploadStatus();
		this.transferRate = transferRate;
		this.timeout = timeout;
		String contentType = request.getContentType();
		String sectionBoundary = "--"+contentType.substring(9+contentType.indexOf("boundary="));
		this.boundary = sectionBoundary.getBytes();
		int contentLength = request.getContentLength();
		this.progress = new FileUploadProgress(contentLength);
		try
		{
			this.in = request.getInputStream();
			// Find first boundary, which is the start of the POST contents.
			readToBoundary(boundary);
		}
		catch (IOException ex)
		{
			progress.setException(ex);
			throw ex;
		}
	}

	/**
		Get progress information about the upload.
	*/
	public FileUploadProgress getProgress()
	{
		return progress;
	}

	/**
		Check if the end of the upload has been reached or not.
		@return TRUE if the end has been reached, FALSE otherwise
	*/
	public boolean eof()
	{
		return status.eof;
	}

	/**
		Read from the input stream until a file section is found or
		the end is encountered.
		@return An {@link UploadedFile} object with information about the next file
			section, or null if there are no more file sections
		@throws IOException If reading the file fails.
		@throws UploadAbortedException If the uploaded has been requested to abort.
	*/
	public UploadedFile next()
		throws IOException, UploadAbortedException
	{
		try
		{
			while (!status.eof)
			{
				// Read the headers of the next section
				SectionHeaders headers = readSectionHeaders();
				if (headers.filename != null)
				{
					// File section found
					UploadedFile uf = new UploadedFile(this, headers.filename, headers.mimeType);
					addFile(headers.name, uf);
					return uf;
				}
				else if (headers.name != null)
				{
					// Form data section
					String formData = readFormData(boundary);
					addParameter(headers.name, formData);
				}
				else
				{
					// Don't understand the headers, skip to next section
					readToBoundary(boundary);
				}
			}
			in.close();
			return null;
		}
		catch (IOException ex)
		{
			progress.setException(ex);
			throw ex;	
		}
	}


	/**
		Upload a single file section to the specified output stream
	*/
	void doUpload(OutputStream out, UploadedFile uf)
		throws IOException, UploadAbortedException
	{
		InputStream in = getInputStream(uf);
		FileUtil.copy(in, out);
	}

	/**
		Get an input stream for reading from the current file section.
	*/
	InputStream getInputStream(UploadedFile uf)
		throws IOException
	{
		progress.setCurrentFilename(uf.getFilename());
		return new UploadStream();
	}
	

/*
        Content-type: multipart/form-data, boundary=AaB03x

        --AaB03x
        content-disposition: form-data; name="field1"

        Joe Blow
        --AaB03x
        content-disposition: form-data; name="pics"
        Content-type: multipart/mixed, boundary=BbC04y

        --BbC04y
        Content-disposition: attachment; filename="file1.txt"
        Content-Type: text/plain

        ... contents of file1.txt ...
        --BbC04y
        Content-disposition: attachment; filename="file2.gif"
        Content-type: image/gif
        Content-Transfer-Encoding: binary

          ...contents of file2.gif...
        --BbC04y--
        --AaB03x--

*/
/*
        Content-type: multipart/form-data, boundary=AaB03x

        --AaB03x
        content-disposition: form-data; name="field1"

        Joe Blow
        --AaB03x
        content-disposition: form-data; name="pics"; filename="file1.txt"
        Content-Type: text/plain

         ... contents of file1.txt ...
        --AaB03x--
*/


	/**
		Store the value of a regular form field.
		@param name The name of the form field
		@param value The value
	*/
	private void addParameter(String name, String value)
	{
		List<String> a = parameters.get(name);
		if (a == null)
		{
			a = new LinkedList<String>();
			parameters.put(name, a);
		}
		a.add(value);
	}

	/**
		Store information about an uploaded file.
		@param name The name of the form field
		@param file A <code>UploadedFile</code> object containing information 
			about the uploaded file
	*/
	private void addFile(String name, UploadedFile file)
	{
		List<UploadedFile> a = files.get(name);
		if (a == null)
		{
			a = new LinkedList<UploadedFile>();
			files.put(name, a);
		}
		a.add(file);
	}

	/**
		Get the value for a form field. Use this method if the form contains only a single
		element with this name, otherwise use the {@link #getParameterValues(String)}
		method.
		@param name The name of the form field
		@return The value of form field or NULL if it is not found
		@see #getParameterValues(String)
	*/
	public String getParameter(String name)
	{
		List<String> a = parameters.get(name);
		if (a == null || a.size() == 0)
		{
			return null;
		}
		else
		{
			return a.get(0);
		}
	}

	/**
		Get an array of values for a form field. Use this method if the form contains several
		elements with the same name, otherwise use the {@link #getParameter(String)}
		method.
		@param name The name of the form field
		@return An array of values, or NULL if no values are found
		@see #getParameter(String)
	*/
	public String[] getParameterValues(String name)
	{
		List<String> a = parameters.get(name);
		if (a == null || a.size() == 0)
		{
			return null;
		}
		else
		{
			String[] values = new String[a.size()];
			a.toArray(values);
			return values;
		}
	}

	/**
		Get the names for all fields used in the form, excluding file input fields.
		@return An <code>Iterator</code> object that can be used to iterate through
			the collection of form fields
	*/
	public Iterator getParameterNames()
	{
		return Collections.unmodifiableSet(parameters.keySet()).iterator();
	}
	
	/**
		Get the file information for an uplodaded file. Use this method if the form contains only a single
		element with this name, otherwise use the {@link #getFileValues(String)}
		method.
		@param name The name of the upload form field
		@return An <code>UploadedFile</code> object or NULL if it the field with this name is not found
		@see #getFileValues(String)
	*/
	public UploadedFile getFile(String name)
	{
		List<UploadedFile> a = files.get(name);
		if (a == null || a.size() == 0)
		{
			return null;
		}
		else
		{
			return a.get(0);
		}
	}

	/**
		Get the file information for uplodaded files. Use this method if the form contains several
		elements with the same name, otherwise use the {@link #getFile(String)}
		method.
		@param name The name of the upload form field(s)
		@return An array of <code>UploadedFile</code> objects or NULL if it no fields with
			this name are not found
		@see #getFile(String) getFile
	*/
	public UploadedFile[] getFileValues(String name)
	{
		List<UploadedFile> a = files.get(name);
		if (a == null || a.size() == 0)
		{
			return null;
		}
		else
		{
			UploadedFile[] files = new UploadedFile[a.size()];
			a.toArray(files);
			return files;
		}
	}

	/**
		Set exception information on the associated {@link FileUploadProgress}
		object. This is a useful way to communicate if the uploading is done 
		by one thread and the progress display in another. This method is also called 
		if an exception occures internally in this object.
	 	@param t The throwable object to set. 
	*/
	public void setException(Throwable t)
	{
		progress.setException(t);
	}

	private static final Pattern NAME_PATTERN = Pattern.compile(".* name=\"([^\"]*)\".*");
	private static final Pattern FILENAME_PATTERN = Pattern.compile(".* filename=\"([^\"]*)\".*");

	private String getHeaderAttribute(String header, Pattern p)
	{
		Matcher m = p.matcher(header);
		return m.matches() ? m.group(1) : null;
	}

	/**
		Check if the given <code>line</code> matches the <code>boundary</code>.
		The <code>line</code> always has an extra CRLF pair at the end and may also have
		two dashes (--). The two dashes indicates the end of the input for the form.
	*/
	private void checkBoundary(byte[] line, int lineLength, byte[] boundary, int boundaryLength)
	{
		if (!(lineLength == boundaryLength+2 || lineLength == boundaryLength+4))
		{
			status.atBoundary = false;
			status.atLastBoundary = false;
			return;
		}
		int i = 0;
		while (i < boundaryLength && line[i] == boundary[i])
		{
			i++;
		}
		status.atBoundary = (i == boundaryLength);
		status.atLastBoundary = (status.atBoundary && (line[i] == '-') && (line[i+1] == '-'));
		return;
	}

	/**
		Reads the next line of input and returns it as a string. We assume that the length of
		the line is less than 1000 bytes, or this method will clip the string after that many characters.
		This method is used when reading section headers.
		@return The string
	*/
	private String readLineAsString()
		throws IOException
	{
		byte[] line = new byte[1000];
		int bytes = readLine(line, 0, line.length);
		if (bytes != -1)
		{
			progress.addTransferredBytes(bytes);
			if (bytes >= 3)
			{
				return new String(line, 0, bytes - 2);	// -2 because we strip CRLF at the end of the line.
			}
			else
			{
				return EMPTY_STRING;
			}
		}
		else
		{
			return EMPTY_STRING;
		}
	}

	/**
		Read from ServletInputStream until the boundary is found. This method is used to read
		data from ordinary (not file upload) form fields.
		@return A string representation of the data that was read
	*/
	private String readFormData(byte[] boundary)
		throws IOException, UploadAbortedException
	{
		int bytes = 0;
		int boundaryLength = boundary.length;
		byte[] line = new byte[1000];
		StringBuffer sb = new StringBuffer();
		status.atBoundary = false;
		status.atLastBoundary = false;

		while (!status.eof && !status.atBoundary)
		{
			if (progress.hasAborted()) throw new UploadAbortedException();
			bytes = readLine(line, 0, line.length);
			if (bytes != -1)
			{
				progress.addTransferredBytes(bytes);
				checkBoundary(line, bytes, boundary, boundaryLength);
				if (!status.atBoundary)
				{
					sb.append(new String(line, 0, bytes));
				}
			}
		}
		if (sb.length() >= 3)
		{
			return sb.substring(0, sb.length()-2);// -2 because we strip CRLF at the end of the last line.
		}
		else
		{
			return EMPTY_STRING;
		}
	}

	/**
		Read from ServletInputStream until the boundary is found
		on a line by itself. This method is used to skip data whenver necessary.
	*/
	private void readToBoundary(byte[] boundary)
		throws IOException, UploadAbortedException
	{
		int bytes = 0;
		int boundaryLength = boundary.length;
		byte[] line = new byte[1000];
		status.atBoundary = false;
		status.atLastBoundary = false;

		while (!status.eof && !status.atBoundary)
		{
			if (progress.hasAborted()) throw new UploadAbortedException();
			bytes = readLine(line, 0, line.length);
			if (bytes != -1)
			{
				progress.addTransferredBytes(bytes);
				checkBoundary(line, bytes, boundary, boundaryLength);
			}
		}
	}

	/**
		Read from ServletInputStream until an empty line is found. This method is used to read
		section headers.
		
		return A <code>SectionHeaders</code> object with information about the headers found
	*/
	private SectionHeaders readSectionHeaders()
		throws IOException, UploadAbortedException
	{
		// Get headers for the next section
		String name = null;
		String filename = null;
		String mimeType = null;
		String header = readLineAsString();
		while ((!status.eof) && (!(header == EMPTY_STRING)))
		{
			if (progress.hasAborted()) throw new UploadAbortedException();
			if (header.startsWith("Content-Disposition: "))
			{
				name = getHeaderAttribute(header, NAME_PATTERN);
				filename = getHeaderAttribute(header, FILENAME_PATTERN);
				int i = filename != null ? filename.lastIndexOf("\\") : -1;
				if (i >= 0) filename = filename.substring(i+1);
			}
			if (header.startsWith("Content-Type: "))
			{
				mimeType = header.substring("Content-Type: ".length());
			}
			header = readLineAsString();
		}
		return new SectionHeaders(name, mimeType, filename);
	}

	/**
		Check if upload has been aborted. Check the transfer rate and wait if necessary.
	*/
	private void checkAbortAndDelay()
		throws UploadAbortedException
	{
		if (progress.hasAborted()) throw new UploadAbortedException();
		// Do we need to check the transfer rate?
		if (transferRate > 0 && progress != null)
		{
			// Yes, have we transferred too much in too short time?
			if (progress.getTransferredBytes() > transferRate * progress.getMillisecondsSinceStart() / 1000)
			{
				// Yes, how long should we wait? Delay is milliseconds
				long delay = 1000 * progress.getTransferredBytes() / transferRate - progress.getMillisecondsSinceStart();
				if (delay > 1000)
				{
					try
					{
						java.lang.Thread.sleep(delay);
					}
					catch (InterruptedException ex)
					{}
				}
			}
		}
	}

	/**
		Read next line of data into a buffer.
		@return The number of bytes read or -1 if the end is reached
		@since 2.1.2
	*/
	private int readLine(byte[] b, int off, int len)
		throws IOException
	{
		int bytes = 0;
		boolean gotData = false;
		long startTime = System.currentTimeMillis();
		while (!gotData)
		{
			try
			{
				bytes = in.readLine(b, off, len);
				gotData = true;
			}
			catch (SocketTimeoutException ex)
			{
				long waitTime = System.currentTimeMillis() - startTime;
				log.debug("SocketTimeoutException after " + waitTime + "ms", ex);
				if (waitTime > timeout)
				{
					SocketTimeoutException ex2 = 
						new SocketTimeoutException("No data for " + (waitTime / 1000) + " seconds");
					ex2.initCause(ex);
					log.debug("Total timeout exceeded after " + waitTime + "ms", ex2);
					throw ex2;
				}
				else
				{
					log.debug("Sleeping 1000ms");
					try
					{
						Thread.sleep(1000);
					}
					catch (Throwable t)
					{}
				}
			}
		}
		if (bytes == -1)
		{
			status.eof = true;
			in.close();
		}
		return bytes;
	}
	
	
	/**
		This class contains information extracted from the section headers for a form field
		or file upload field.
	*/
	private static class SectionHeaders
	{
		private final String name;
		private final String mimeType;
		private final String filename;

		private SectionHeaders(String name, String mimeType, String filename)
		{
			this.name = name;
			this.mimeType = mimeType;
			this.filename = filename;
		}
	}
	
	/**
		This class contains information about the read status on the ServletInputStream
	*/
	private static class UploadStatus
	{
		/**
			TRUE if the end of the stream is reached, FALSE otherwise
		*/
		private boolean eof = false;
		
		/**
			TRUE if the {@link #checkBoundary(byte[], int, byte[], int)}
			method finds that the line matches the boundary, FALSE otherwise.
		*/
		private boolean atBoundary = false;
		
		/**
			TRUE if the {@link #checkBoundary(byte[], int, byte[], int)}
			method finds that the line matches the boundary + two extra dashes (--), FALSE otherwise.
		*/
		private boolean atLastBoundary = false;
		
		/**
			Create a new <code>UploadStatus</code> object.
		*/
		private UploadStatus()
		{}
	}

	private class UploadStream
		extends InputStream
	{
		
		private byte[] buffer;
		private byte[] nextBuffer;
		private int start;
		private int end;
		private int nextEnd;

		private UploadStream()
			throws IOException
		{
			this.buffer = new byte[1000];
			this.nextBuffer = new byte[1000];
			this.start = 0;
			this.end = 0;
			this.nextEnd = 0;
			status.atBoundary = false;
			readMoreData();
			readMoreData();
		}
		
		/*
			From the InputStream class
			-------------------------------------------
		*/
		@Override
		public int available() 
			throws IOException
		{
			return end - start;
		}
		/**
			Not supported.
		*/
		@Override
		public synchronized void mark(int readlimit)
		{}
		/**
		 	Not supported.
		*/
		@Override
		public boolean markSupported()
		{
			return false;
		}
		public int read() 
			throws IOException
		{
			int available = end - start;
			if (available == 0)
			{
				if (status.eof || status.atBoundary) return -1;
				available = readMoreData();
			}
			int result = -1;
			if (available > 0)
			{
				// Important! Mask with 0xff to get values in range (0,255), NOT (-128,127)
				result = buffer[start] & 0xff;
				start++;
			}
			return result;
		}
		
		@Override
		public int read(byte[] b, int off, int len) 
			throws IOException
		{
			int available = end - start;
			if (available == 0 && (status.eof || status.atBoundary)) return -1;
			int copied = 0;
			int remain = len;
			if (available == 0) 
			{
				available = readMoreData();
			}
			while (remain > 0 && available > 0)
			{
				if (available >= remain)
				{
					System.arraycopy(buffer, start, b, off, remain);
					start += remain;
					copied += remain;
					available -= remain;
					remain = 0;
				}
				else
				{
					if (available > 0)
					{
						System.arraycopy(buffer, start, b, off, available);
						off += available;
						remain -= available;
						copied += available;
					}
					available = readMoreData();
				}
			}
			return copied;
		}
		
		@Override
		public int read(byte[] b) 
			throws IOException
		{
			return read(b, 0, b.length);
		}
		/**
			Not supported.
		*/
		@Override
		public synchronized void reset() 
			throws IOException
		{}
		// -------------------------------------------
	
		/**
			Read the next line from the ServletInputStream until
			the boundary is found on a line by itself. This method is used to read data
			from file upload fields.
		*/
		private int readMoreData()
			throws IOException, UploadAbortedException
		{
			if (status.eof || status.atBoundary) 
			{
				start = 0;
				end = 0;
				return 0;
			}
			checkAbortAndDelay();

			// Rotate the buffers
			byte[] oldBuffer = buffer;
			buffer = nextBuffer;
			start = 0;
			end = nextEnd;
			nextBuffer = oldBuffer;
			
			int boundaryLength = boundary.length;
			int bytes = readLine(nextBuffer, 0, nextBuffer.length);

			if (bytes != -1)
			{
				progress.addTransferredBytes(bytes);
				checkBoundary(nextBuffer, bytes, boundary, boundaryLength);
				if (status.atBoundary)
				{
					end -= 2; // The last crlf is not part of the data
					nextEnd = 0;
				}
				else
				{
					nextEnd = bytes;
				}
			}
			return end;
		}
	}
}
