/*
	$Id: ButtonGroup.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib;

import java.util.List;
import java.util.LinkedList;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

/**
	This tag is used to display a clickable button.
	<p>
	<b>Example:</b>
	<pre>
   &lt;%@ taglib prefix="base" uri="/WEB-INF/base.tld" %&gt;
   &lt;base:button onclick="alert('click')" title="Click here"&gt;
   &lt;/base:note&gt;
</pre>

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;base:buttongroup
      id=...
      class=...
      style=...
      align=...
      &gt;
      &lt;base:button ... &gt;
      &lt;base:button ... &gt;
	&lt;/base:buttongroup&gt;
</pre>
	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>no</td>
		<td>The ID of the button. The value of this attribute
		goes directly into the standard <code>id</code> attribute.
		This means that if you need a reference to the button object in your own
		JavaScript code, you may for example use the following code:
		<p>
		<code>
		buttonGroup = document.getElementById('&lt;id&gt;');
		</code>
		</td>
	</tr>
	<tr>
		<td>clazz</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different buttons.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different buttons.
		</td>
	</tr>
	<tr>
		<td>align</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>align</code> attribute. This allows you to apply different styles
			to different buttons.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.0
*/
public class ButtonGroup
	extends TagSupport
{

	/**
		
	*/
	private static final long serialVersionUID = -8808469470232672680L;

	/**
		Optional <code>class</code> attribute
	*/
	private String clazz = "buttongroup";
	
	/**
		Optional <code>style</code> attribute
	*/
	private String style = null;
	
	/**
		Optional <code>align</code> attribute
	*/
	private String align = null;

	/**
		The buttons in this group.
	*/
	private List<String> buttons;
	
	/*
		Taglib initialization methods
	*/
	public void setClazz(String clazz)
	{
		this.clazz = clazz;
	}
	public String getClazz()
	{
		return clazz;
	}

	public void setStyle(String style)
	{
		this.style = style;
	}
	public String getStyle()
	{
		return style;
	}
	
	public void setAlign(String align)
	{
		this.align = align;
	}
	public String getAlign()
	{
		return align;
	}

	void addButton(String html)
	{
		buttons.add(html);
	}
	
	public int doStartTag()
		throws JspException
	{
		buttons = new LinkedList<String>();
		return EVAL_BODY_INCLUDE;
	}
	
	public int doEndTag()
		throws JspException
	{
		StringBuilder sb = new StringBuilder();
		int numButtons = buttons.size();
		if (numButtons > 0)
		{
			int percent = 100 / numButtons;
			sb.append("<div");
			if (getId() != null) sb.append(" id=\"").append(getId()).append("\"");
			if (getClazz() != null) sb.append(" class=\"").append(getClazz()).append("\"");
			if (getStyle() != null) sb.append(" style=\"").append(getStyle()).append("\"");
			sb.append(">\n");
			sb.append("<table");
			if (getAlign() != null) sb.append(" align=\"").append(getAlign()).append("\"");
			sb.append(">\n");
			sb.append("<tr>\n");
			for (String b : buttons)
			{
				sb.append("<td width=\"").append(percent).append("%\">").append(b).append("</td>\n");
			}
			sb.append("</tr>\n");
			sb.append("</table>\n");
			sb.append("</div>\n");
		}
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}

}




