/**
	$Id: AnyToAny.java 5021 2009-06-26 12:07:15Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.mapping.PersistentClass;
import org.hibernate.mapping.Table;

import net.sf.basedb.core.Transactional.Action;
import net.sf.basedb.core.data.AnyToAnyData;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.core.signal.ThreadSignalHandler;

/**
	An any-to-any link is a named, directed link from one item to another item.
	The items can be of any type. Permission-wise the links belong to the start 
	item. Ie. to be able create, update and delete links a user must have 
	WRITE permission on the start item. The user must also have USE permission 
	on the item we are linking to.
	<p>
	All any-to-any links from an item has a name which must be unique for that
	item.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2009-06-26 14:07:15 +0200 (Fri, 26 Jun 2009) $
*/
public class AnyToAny
	extends BasicItem<AnyToAnyData>
	implements Nameable
{

	/**
		The type of item represented by this class.
		@see Item#ANYTOANY
		@see #getType()
	*/
	public static final Item TYPE = Item.ANYTOANY;

	/**
		Create a new <code>AnyToAny</code> link
	
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param from The item to create the link from
		@param to The item to link to
		@param name The name of the link, must be unique among all links for the from
			item
		@param usingTo TRUE if the link counts as using the <code>to</code> item, 
			FALSE otherwise (see {@link BasicItem#isUsed()})
		@return The new <code>AnyToAny</code> link
		@throws InvalidDataException If any of the items are null, or if the
			items are equal or if any of the items is an AnyToAny item
		@throws BaseException If there is an error
	*/
	public static AnyToAny getNew(DbControl dc, BasicItem from, BasicItem to, String name, boolean usingTo)
		throws InvalidDataException
	{
		AnyToAny ata = dc.newItem(AnyToAny.class);
		if (from != null && from.equals(to)) 
		{
			throw new InvalidDataException("Can't link an item to itself: " + from);
		}
		ata.setName(name);
		ata.setFrom(from);
		ata.setTo(to);
		ata.setUsingTo(usingTo);
		return ata;
	}

	/**
		Get an <code>AnyToAny</code> link when you know the id.
	
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param id The id of the item to load
		@return The <code>AnyToAnyLink</code> item
		@throws ItemNotFoundException If an item with the specified 
			id is not found
		@throws PermissionDeniedException If the logged in user doesn't 
			have read permission to the item
		@throws BaseException If there is another error
	*/
	public static AnyToAny getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		AnyToAny ata = dc.loadItem(AnyToAny.class, id);
		if (ata == null) throw new ItemNotFoundException("AnyToAny[id="+id+"]");
		return ata;
	}

	/**
		Get an <code>AnyToAny</code> link when you know the item 
		it is linking from and the name of the link.
	
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param from The item the link is linking from
		@param name The name of the link
		@return The <code>AnyToAnyLink</code> item
		@throws InvalidDataException If the from or name parameter is null
		@throws ItemNotFoundException If a link with the specified 
			name is not found
		@throws BaseException If there is another error
	*/
	public static AnyToAny getByName(DbControl dc, BasicItem from, String name)
		throws InvalidDataException, ItemNotFoundException, BaseException
	{
		if (from == null) throw new InvalidUseOfNullException("from");
		if (name == null) throw new InvalidUseOfNullException("name");
		AnyToAny ata = dc.getItem(AnyToAny.class, 
			getDataByName(dc.getHibernateSession(), from.getId(), from.getType(), name));
		if (ata == null) throw new ItemNotFoundException("AnyToAny[name="+name+"]");
		return ata;
	}
	
	/**
		Get an any-to-any link.
		@param session An open Hibernate session
		@param fromId The id of the source item
		@param fromType The type of the source item
		@param name The name of the link
		@return The linked item, or null if no link is found
		@since 2.9
	*/
	static AnyToAnyData getDataByName(org.hibernate.Session session, int fromId, Item fromType, String name)
	{
		name = name.trim();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_ANYTOANY_FOR_NAME");
		/*
			SELECT ata
			FROM AnyToAnyData ata
			WHERE ata.fromId = :fromId AND ata.fromType = :fromType AND ata.name = :name
		*/
		query.setInteger("fromId", fromId);
		query.setInteger("fromType", fromType.getValue());
		query.setString("name", name);
		AnyToAnyData ata = HibernateUtil.loadData(AnyToAnyData.class, query);
		return ata;
	}
	
	/**
		Get a new or existing <code>AnyToAny</code> link. If a link
		already exists it's destination item will be updated to link
		to the <code>to</code> parameter. If a new link was created
		you must call {@link DbControl#saveItem(BasicItem)} to save 
		the link in the database. Otherwise, you don't have to do anything.
		
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param from The item to create/get the link from
		@param to The item to link to
		@param name The name of the link, must be unique among all links for the from
			item
		@param usingTo TRUE if the link counts as using the <code>to</code> item, 
			FALSE otherwise (see {@link BasicItem#isUsed()})
		@return The new or existing <code>AnyToAny</code> link
		@throws InvalidDataException If any of the items are null, or if the
			items are equal or if any of the items is an AnyToAny item
		@throws BaseException If there is an error
	*/
	public static AnyToAny getNewOrExisting(DbControl dc, BasicItem from, String name, BasicItem to, boolean usingTo)
	{
		AnyToAny ata = null;
		try
		{
			ata = getByName(dc, from, name);
			ata.setTo(to);
			ata.setUsingTo(usingTo);
		}
		catch (ItemNotFoundException ex)
		{
			ata = getNew(dc, from, to, name, usingTo);
		}
		return ata;
	}
	
	/**
		Get a query that returns all links linking from the specified item.
		@param from The source of the links
		@return An <code>ItemQuery</code> object
		@throws InvalidDataException If the from parameter is null
	*/
	public static ItemQuery<AnyToAny> getLinksFrom(BasicItem from)
		throws InvalidDataException
	{
		if (from == null) throw new InvalidUseOfNullException("from");
		ItemQuery<AnyToAny> query = new ItemQuery<AnyToAny>(AnyToAny.class, null);
		query.restrictPermanent(
			Restrictions.and(
				Restrictions.eq(
					Hql.property("fromId"),
					Expressions.integer(from.getId())
				),
				Restrictions.eq(
					Hql.property("fromType"),
					Expressions.integer(from.getType().getValue())
				)
			)
		);
		return query;
	}

	/**
		Get a query that returns all links linking to the specified item.
		@param to The destination of the links
		@return An <code>ItemQuery</code> object
		@throws InvalidDataException If the to parameter is null
	*/
	public static ItemQuery<AnyToAny> getLinksTo(BasicItem to)
	{
		if (to == null) throw new InvalidUseOfNullException("to");
		ItemQuery<AnyToAny> query = new ItemQuery<AnyToAny>(AnyToAny.class, null);
		query.restrictPermanent(
			Restrictions.and(
				Restrictions.eq(
					Hql.property("toId"),
					Expressions.integer(to.getId())
				),
				Restrictions.eq(
					Hql.property("toType"),
					Expressions.integer(to.getType().getValue())
				)
			)
		);
		return query;
	}
	
	/**
		@param dc DbControl to use when accessing the database 
	 	@param from BasicItem to unlink from.
	  	@param name Name of the link. 
	  	@throws PermissionDeniedException If the logged in user doesn't have
			WRITE permission for the item
		@throws InvalidDataException If the from or name parameter is null
		@throws BaseException If there is another error
	*/
	public static void unlinkFrom(DbControl dc, BasicItem from, String name)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		if (from == null) throw new InvalidUseOfNullException("from");
		if (name == null) throw new InvalidUseOfNullException("name");
		from.checkPermission(Permission.WRITE);
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(dc.getHibernateSession(), 
			"DELETE_ANYTOANY_FROM");
		/*
			DELETE FROM AnyToAnyData ata 
			WHERE ata.fromId = :fromId AND ata.fromType = :fromType AND ata.name = :name
		*/
		query.setInteger("fromId", from.getId());
		query.setInteger("fromType", from.getType().getValue());
		query.setString("name", name);
		HibernateUtil.executeUpdate(query);
	}
	
	/**
		Delete all links linking from the specified item.
		
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@param from The source of the links
		@return The number of deleted links
		@throws PermissionDeniedException If the logged in user doesn't have
			WRITE permission for the item
		@throws InvalidDataException If the from parameter is null
		@throws BaseException If there is another error
	*/
	public static int unlinkAllFrom(DbControl dc, BasicItem from)
		throws PermissionDeniedException, InvalidDataException, BaseException
	{
		if (from == null) throw new InvalidUseOfNullException("from");
		from.checkPermission(Permission.WRITE);

		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(dc.getHibernateSession(), 
			"DELETE_ALL_ANYTOANY_FROM");
		/*
			DELETE FROM AnyToAnyData ata 
			WHERE ata.fromId = :fromId AND ata.fromType = :fromType
		*/
		query.setInteger("fromId", from.getId());
		query.setInteger("fromType", from.getType().getValue());
		return HibernateUtil.executeUpdate(query);
	}
	
	
	/**
		Check if an item has a link with a given name.
	
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param from The item the link is linking from
		@param name The name of the link
		@return TRUE if the link exists, FALSE otherwise
		@throws InvalidDataException If the from or name parameter is null
		@throws BaseException If there is another error
		@since 2.4
	*/
	public static boolean exists(DbControl dc, BasicItem from, String name)
		throws InvalidDataException, BaseException
	{
		if (from == null) throw new InvalidUseOfNullException("from");
		if (name == null) throw new InvalidUseOfNullException("name");
		name = name.trim();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(dc.getHibernateSession(), 
			"GET_ANYTOANY_FOR_NAME");
		/*
			SELECT ata
			FROM AnyToAnyData ata
			WHERE ata.fromId = :fromId AND ata.fromType = :fromType AND ata.name = :name
		*/
		query.setInteger("fromId", from.getId());
		query.setInteger("fromType", from.getType().getValue());
		query.setString("name", name);
		AnyToAny ata = dc.getItem(AnyToAny.class, HibernateUtil.loadData(AnyToAnyData.class, query));
		return ata != null;
	}
	
	/**
		@deprecated Use {@link #deleteStrayLinks(ProgressReporter)} instead.
	*/
	public static synchronized void deleteStrayLinks()
	{
		deleteStrayLinks(null);
	}

	/**
		Delete all links that are linking to non-existing items. This method 
		is intended to be executed at regular intervals by a cleanup application.
		Although the {@link BasicItem#onBeforeCommit(Transactional.Action)} method
		tries to delete as many links as it can, it will not not delete links
		leading to items that have been deleted by Hibernate cascade.
		@param progress An optional progress reporter
		@since 2.13
	*/
	public static synchronized int deleteStrayLinks(ProgressReporter progress)
	{
		org.hibernate.Session session = null;
		org.hibernate.Transaction tx = null;
		int numDeleted = 0;

		try
		{
			session = HibernateUtil.newSession();
			tx = HibernateUtil.newTransaction(session);
			
			// Find the used items types in any-to-any links
			if (progress != null) 
			{
				progress.display(5, "Finding item types used by any-to-any links...");
			}
			org.hibernate.Query query = HibernateUtil.getPredefinedSQLQuery(session, 
					"FIND_USED_TYPES_IN_ANYTOANY");
			/*
				-- NOTE! UNION produces a distinct result by default (which is what we want)
				SELECT [a].[from_type] FROM [AnyToAny] [a]
				UNION 
				SELECT [a].[to_type] FROM [AnyToAny] [a]
			*/
			List<Integer> itemTypes = HibernateUtil.loadList(Integer.class, query, null);
			
			// For each item type; find and delete non-existing items
			int index = 0;
			int numItemTypes = itemTypes.size();
			org.hibernate.Query deleteQuery = 
				HibernateUtil.getPredefinedQuery(session, "DELETE_STRAY_ANYTOANY");
			for (int itemType : itemTypes)
			{
				ThreadSignalHandler.checkInterrupted();
				Item item = Item.fromValue(itemType);
				if (progress != null)
				{
					int percent = 10 + (90 * index) / numItemTypes;
					progress.display(percent, "Deleting any-to-any links referencing non-existing item: " + item);
				}
				PersistentClass pClass = HibernateUtil.getClassMapping(item.getDataClass().getName());
				Table table = pClass.getTable();
			
				query = HibernateUtil.getPredefinedSQLQuery(session, "SELECT_STRAY_ANYTOANY", 
						table.getName());
				/*
					SELECT [a].[id]
					FROM [AnyToAny] [a] 
					LEFT JOIN [{1}] [t] 
						ON ([t].[id] = [a].[from_id] and [a].[from_type] = :type)
						OR ([t].[id] = [a].[to_id] and [a].[to_type] = :type) 
					WHERE [t].[id] is null AND ([a].[from_type = :type OR [a].[to_type] = :type)
				*/
				query.setInteger("type", itemType);
				List<Integer> stray = HibernateUtil.loadList(Integer.class, query, null);
				if (stray.size() > 0)
				{
					deleteQuery.setParameterList("ids", stray, Hibernate.INTEGER);
					numDeleted += HibernateUtil.executeUpdate(deleteQuery);
				}
				index++;
			}
			HibernateUtil.commit(tx);
		}
		catch (RuntimeException ex)
		{
			if (tx != null) HibernateUtil.rollback(tx);
			throw ex;
		}
		finally
		{
			if (session != null) HibernateUtil.close(session);
		}
		return numDeleted;
	}
	
	private BasicItem from;
	private BasicItem to;
	
	/**
	 	Creates a new link item from the given data.
	*/
	AnyToAny(AnyToAnyData data)
	{
		super(data);
	}
	
	/**
		Special constructor when the from item is known.
	*/
	AnyToAny(AnyToAnyData data, BasicItem from)
	{
		super(data);
		this.from = from;
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------

	/*
		From BasicItem
		-------------------------------------------
	*/
	/**
		Always FALSE.
	*/
	@Override
	public boolean isUsed()
		throws BaseException
	{
		return false;
	}
	
	/**
		We may have to set the {@link #getFromId()} and/or {@link #getToId()}
		values if those items are also created in the same transaction.
	*/
	@Override
	void onBeforeCommit(Action action) 
		throws BaseException
	{
		super.onBeforeCommit(action);
		if (action == Action.CREATE)
		{
			// In case either the to or from items are also new items
			int fromId = getFrom().getId();
			if (fromId == 0) throw new ItemNotFoundException(getFromType()+"[id="+fromId+"]");
			getData().setFromId(fromId);
			int toId = getTo().getId();
			if (toId == 0) throw new ItemNotFoundException(getToType()+"[id="+toId+"]");
			getData().setToId(toId);
		}
	}

	/**
		Permissions are granted based on the permission on the 
		from item. READ permission gives READ permission. WRITE permission
		gives CREATE, WRITE and DELETE permissions.
	*/
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		BasicItem from = null;
		try
		{
			from = getFrom();
		}
		catch (PermissionDeniedException ex)
		{}
		if (from != null && from.hasPermission(Permission.WRITE))
		{
			granted |= Permission.grant(Permission.CREATE, Permission.READ, Permission.WRITE, Permission.DELETE);				
		}
		else if (from != null && from.hasPermission(Permission.READ))
		{
			granted |= Permission.grant(Permission.READ);
		}
		super.initPermissions(granted, denied);
	}
	PluginPermission getPluginPermissions()
	{
		return PluginPermission.NO_PLUGIN;
	}
	// -------------------------------------------
	
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	public String getName()
	{
		return getData().getName();
	}
	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setName(getData(), name);
	}
	public String getDescription()
	{
		return getData().getDescription();
	}
	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------
	
	private void setFrom(BasicItem from)
		throws InvalidDataException, PermissionDeniedException
	{
		if (from == null) throw new InvalidUseOfNullException("from");
		from.checkPermission(Permission.WRITE);
		if (from.getType() == TYPE) throw new InvalidDataException("Can't link to an any-to-any link: " + from);
		getData().setFromId(from.getId());
		getData().setFromType(from.getType().getValue());
		this.from = from;
	}

	/**
		Get the ID of the item this link is linking from.
		@return The ID
	*/
	public int getFromId()
	{
		return getData().getFromId();
	}
	
	/**
		Get the type of the item this link is linking from.
		@return The type of item
	*/
	public Item getFromType()
	{
		return Item.fromValue(getData().getFromType());
	}
	
	/**
		Get the item this link is linking from.
		@return The <code>BasicItem</code> item
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission to the item
		@throws BaseException If there is another error
	*/
	public BasicItem getFrom()
		throws PermissionDeniedException, BaseException
	{
		if (from == null)
		{	
			from = getFromType().getById(getDbControl(), getFromId());
		}
		return from;
	}
	
	/**
		Get the ID of the item this link is linking to.
		@return The ID
	*/
	public int getToId()
	{
		return getData().getToId();
	}
	
	/**
		Get the type of the item this link is linking to.
		@return The type of the item
	*/
	public Item getToType()
	{
		return Item.fromValue(getData().getToType());
	}

	/**
		Get the item this link is linking to.
		@return The <code>BasicItem</code> item
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission to the item
		@throws BaseException If there is another error
	*/
	public BasicItem getTo()
		throws PermissionDeniedException, BaseException
	{
		if (to == null)
		{	
			to = getToType().getById(getDbControl(), getToId());
		}
		return to;
	}

	/**
		Set the destination item of the link.
		@param to The destination item
		@throws InvalidDataException If the item is null or is an <code>AnyToAny</code> link
		@throws PermissionDeniedException If the logged in user doesn't have write permission
			for the link and use permission for the item
	*/
	public void setTo(BasicItem to)
		throws InvalidDataException, PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (to == null) throw new InvalidUseOfNullException("to");
		to.checkPermission(Permission.USE);
		if (to.getType() == TYPE) throw new InvalidDataException("Can't link to an any-to-any link: " + to);
		getData().setToId(to.getId());
		getData().setToType(to.getType().getValue());
		this.to = to;
	}

	
	/**
		If this link counts as using the destination item. This affects the
		possibility to delete items.
		@return TRUE when it is not possible to delete, FALSE otherwise.
	*/
	public boolean isUsingTo()
	{
		return getData().isUsingTo();
	}
	
	/**
		If this link counts as using the destination item. This affects the
		possibility to delete items.
		@param usesTo TRUE if this counts as using the item, FALSE otherwise.
		@throws PermissionDeniedException If the logged in user doesn't
			have write permission
	*/
	public void setUsingTo(boolean usesTo)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setUsingTo(usesTo);
		
	}

	
}
