/*
	$Id: ProtocolType.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Date;
import java.util.Set;

import net.sf.basedb.core.data.ProtocolTypeData;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	This class is used to represent the type of the
	{@link Protocol} items in BASE.

	@author enell
	@version 2.0
	@see Protocol
*/
public class ProtocolType
	extends BasicItem<ProtocolTypeData>
	implements Nameable, Removable, SystemItem, Registered
{

	/**
		The type of item represented by this class.
		@see Item#PROTOCOLTYPE
		@see #getType()
	*/
	public static final Item TYPE = Item.PROTOCOLTYPE;
	
	/**
		The ID for the sampling protocol type, for example a protocol
		used when creating a {@link Sample} from a {@link
		BioSource}.
	*/
	public static final String SAMPLING = "net.sf.basedb.core.ProtocolType.SAMPLING";

	/**
		The ID for the extraction protocol type, for example a protocol
		used when creating an {@link Extract} from a {@link
		Sample}.
	*/
	public static final String EXTRACTION = "net.sf.basedb.core.ProtocolType.EXTRACTION";

	/**
		The ID for the labeling protocol type, for example a protocol
		used when creating a {@link LabeledExtract} from
		an {@link Extract}.
	*/
	public static final String LABELING = "net.sf.basedb.core.ProtocolType.LABELING";

	/**
		The ID for the pooling protocol type, for example a protocol
		used when creating a biomaterial item ({@link Sample},
		{@link Extract} or {@link LabeledExtract})
		from a set of other biomaterial items.
	*/
	public static final String POOLING = "net.sf.basedb.core.ProtocolType.POOLING";

	/**
		The ID for the hybridization protocol type, for example a protocol
		used when creating a {@link Hybridization} from a set of
		{@link LabeledExtract}:s.
	*/
	public static final String HYBRIDIZATION = "net.sf.basedb.core.ProtocolType.HYBRIDIZATION";

	/**
		The ID for the printing protocol type, for example a protocol
		used when printing a batch of array slides using a print robot.
	*/
	public static final String PRINTING = "net.sf.basedb.core.ProtocolType.PRINTING";

	/**
		The ID for the scanning protocol type, for example a protocol
		used for scanning the microarray slide after an experiment.
	*/
	public static final String SCANNING = "net.sf.basedb.core.ProtocolType.SCANNING";

	/**
		The ID for the feature extraction protocol type, for example a
		protocol used for analysing one or more images to create a raw
		data file.
	*/
	public static final String FEATURE_EXTRACTION = "net.sf.basedb.core.ProtocolType.FEATURE_EXTRACTION";
	
	/**
		Create a new <code>FileType</code> item.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@return The new <code>FileType</code> item
		@throws BaseException If there is an error
	*/
	public static ProtocolType getNew(DbControl dc)
		throws BaseException
	{
		ProtocolType pt = dc.newItem(ProtocolType.class);
		pt.setName("New protocol type");
		pt.getData().setEntryDate(new Date());
		return pt;
	}
	
	/**
		Get a <code>ProtocolType</code> object when you know the ID.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>ProtocolType</code> item
		@throws ItemNotFoundException If an item with the specified ID is not found
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#READ} permission to the items
		@throws BaseException If there is an error
	*/
	public static ProtocolType getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		ProtocolType pt = dc.loadItem(ProtocolType.class, id);
		if (pt == null) throw new ItemNotFoundException("ProtocolType[ID="+id+"]");
		return pt;
	}
	
	/**
		Get a query configured to retrieve protocol types.
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<ProtocolType> getQuery()
	{
		return new ItemQuery<ProtocolType>(ProtocolType.class);
	}
	
	/**
		Creates a new protocoltype item.
		
		@param data The data
	*/
	ProtocolType(ProtocolTypeData data)
	{
		super(data);
	}
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	public String getName()
	{
		return getData().getName();
	}

	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setName(getData(), name);
	}

	public String getDescription()
	{
		return getData().getDescription();
	}

	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------
	/*
		From the Removable interface
		-------------------------------------------
	*/
	public boolean isRemoved()
	{
		return getData().isRemoved();
	}
	public void setRemoved(boolean removed)
		throws PermissionDeniedException
	{
		checkPermission(removed ? Permission.DELETE : Permission.WRITE);
		getData().setRemoved(removed);
	}
	// -------------------------------------------
	/*
		From the SystemItem interface
		-------------------------------------------
	*/
	public String getSystemId()
	{
		return getData().getSystemId();
	}
	public boolean isSystemItem()
	{
		return getSystemId() != null;
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Check if:
		<ul>
		<li>A {@link Protocol} of this type exists
		<li>A {@link PlateEventType} is linked to this protocol type
		</ul>
	*/
	public boolean isUsed()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_PROTOCOLS_FOR_PROTOCOLTYPE", "count(*)");
		/*
			SELECT {1}
			FROM ProtocolData p
			WHERE p.protocolType = :protocolType
		*/
		query.setEntity("protocolType", this.getData());
		boolean used = HibernateUtil.loadData(Long.class, query) > 0;
		if (!used)
		{
			query = HibernateUtil.getPredefinedQuery(session, 
				"GET_PLATEEVENTTYPES_FOR_PROTOCOLTYPE", "count(*)");
			/*
				SELECT {1}
				FROM PlateEventTypeData pet
				WHERE pet.protocolType = :protocolType
			*/
			query.setEntity("protocolType", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}
		return used || super.isUsed();
	}
	/**
		Get all:
		<ul>
		<li>{@link Protocol}:s of this type
		<li>{@link PlateEventType}:s using this protocol type
		<ul>
		@since 2.2
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		
		// Protocols
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_PROTOCOLS_FOR_PROTOCOLTYPE", "p.id");
			/*
				SELECT {1}
				FROM ProtocolData p
				WHERE p.protocolType = :protocolType
			*/
		query.setEntity("protocolType", this.getData());
		addUsingItems(using, Item.PROTOCOL, query);
		
		// Plate event types
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_PLATEEVENTTYPES_FOR_PROTOCOLTYPE", "pet.id");
			/*
				SELECT {1}
				FROM PlateEventTypeData pet
				WHERE pet.protocolType = :protocolType
			*/
		query.setEntity("protocolType", this.getData());
		addUsingItems(using, Item.PLATEEVENTTYPE, query);
			
		return using;
}

	/**
		If this is a system protocol type, delete and create permissions are revoked.
	*/
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		if (isSystemItem()) 
		{
			denied |= Permission.deny(Permission.DELETE, Permission.CREATE);
		}
		super.initPermissions(granted, denied);
	}
	// -------------------------------------------

	/**
		Create a new {@link Protocol} item of this type.
		@return The new <code>Protocol</code> object
		@throws PermissionDeniedException This exception is thrown if
			the logged in user doesn't have {@link Permission#USE}
			permission for this protocol type
		@throws BaseException This exception is thrown if there is another
			error
	*/
	public Protocol newProtocol()
		throws PermissionDeniedException, BaseException
	{
		Protocol pr = Protocol.getNew(getDbControl(), this);
		return pr;
	}

	/**
		Get a query that returns all protocols of this type
		@return An {@link ItemQuery} object
	*/
	public ItemQuery<Protocol> getProtocols()
	{
		ItemQuery<Protocol> query = Protocol.getQuery();
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property("protocolType"), 
				Hql.entity(this)
			)
		);
		return query;
	}

}
