/*
  $Id: SoftwareType.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Date;
import java.util.Set;

import net.sf.basedb.core.data.SoftwareTypeData;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	This class is used to represent the type of
	{@link Software Software} items in BASE. Currently, the only type
	of software we keep information about is feature extraction software.
	It is not possible for client applications to create new
	software types or modify existing ones.

	@author enell
	@version 2.0
	@see Software
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public class SoftwareType
	extends BasicItem<SoftwareTypeData>
	implements Nameable, SystemItem, Registered
{
	/**
		The type of item represented by this class.
		@see Item#SOFTWARETYPE ITEM.SOFTWARETYPE
		@see #getType() getType()
	*/
	private static final Item TYPE = Item.SOFTWARETYPE;
	
	/**
		The id for the <code>SoftwareType</code> item representing the
		softwaretype "feature extraction software".
	*/
	public static final String FEATURE_EXTRACTION = "net.sf.basedb.core.SoftwareType.FEATURE_EXTRACTION";
	
	/**
		Get a <code>SoftwareType</code> object when you know the ID.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>SoftwareType</code> item
		@throws ItemNotFoundException This exception is thrown if an
			item with the specified ID is not found
		@throws PermissionDeniedException This exception is thrown if
			the logged in user doesn't have {@link Permission#READ READ}
			permission to the items
		@throws BaseException This exception is thrown if there is another
			error
	 */
	public static SoftwareType getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		SoftwareType st = dc.loadItem(SoftwareType.class, id);
		if (st == null)
		{
			throw new ItemNotFoundException("SoftwareType[ID=" + id + "]");
		}
		return st;
	}
	
	/**
		Get a query configured to retrieve software types.
		
		@return An {@link ItemQuery} object
	 	@throws BaseException If getting the query fails. 
	*/
	public static ItemQuery<SoftwareType> getQuery()
		throws BaseException
	{
		return new ItemQuery<SoftwareType>(SoftwareType.class);
	}

	/**
		Creates a new softwaretype.
		
		@param data the data
	*/
	SoftwareType(SoftwareTypeData data)
	{
		super(data);
	}

	/*
		From the Nameable interface
		-------------------------------------------
	*/
	public String getName()
	{
		return getData().getName();
	}

	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		NameableUtil.setName(getData(), name);
	}

	public String getDescription()
	{
		return getData().getDescription();
	}

	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the SystemItem interface
		-------------------------------------------
	*/
	public String getSystemId()
	{
		return getData().getSystemId();
	}

	public boolean isSystemItem()
	{
		return getSystemId() != null;
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Check if:
		<ul>
		<li>there are any {@link Software} using this type
		</ul>
	*/
	@Override
	public boolean isUsed()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_SOFTWARE_FOR_SOFTWARETYPE", "count(*)");
		/*
			SELECT {1}
			FROM SoftwareData sw
			WHERE sw.softwareType = :softwaretype
		*/
		query.setEntity("softwaretype", this.getData());
		return HibernateUtil.loadData(Long.class, query) > 0 || super.isUsed();
	}
	/**
		Get all:
		<ul>
		<li>{@link Software} of this type
		<ul>
		@since 2.2
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_SOFTWARE_FOR_SOFTWARETYPE", "sw.id");
			/*
				SELECT {1}
				FROM SoftwareData sw
				WHERE sw.softwareType = :softwaretype
			*/
		query.setEntity("softwaretype", this.getData());
		addUsingItems(using, Item.SOFTWARE, query);
		return using;
	}
		
	@Override
	/**
	 If this is a system software type, delete and create permissions are revoked.
	*/
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		if (isSystemItem())
		{
			denied |= Permission.deny(Permission.DELETE, Permission.CREATE);
		}
		super.initPermissions(granted, denied);
	}
	// -------------------------------------------

	/**
		Create a new {@link Software} item of this type.
	 
		@return The new {@link Software} item
		@throws DetachedItemException If this SoftwareType is detached from DbControl.
		@throws PermissionDeniedException This exception is thrown if
			the logged in user doesn't have {@link Permission#CREATE CREATE}
			permission for software items or {@link Permission#USE USE}
			permission for this software type
		@throws BaseException This exception is thrown if there is another
			error
	*/
	public Software newSoftware() 
		throws PermissionDeniedException, BaseException
	{
		checkPermission(Permission.USE);
		Software sw = Software.getNew(getDbControl(), this);
		return sw;
	}
	
	/**
		Get a query that returns all software of this type
		@return An {@link ItemQuery} object
	*/
	public ItemQuery<Software> getSoftware()
	{
		ItemQuery<Software> query = Software.getQuery();
		query.restrictPermanent(
			Restrictions.eq(
				Hql.property("softwareType"), 
				Hql.entity(this)
			)
		);
		return query;
	}
	
}
