/**
	$Id: EntityQueryWrapper.java 4517 2008-09-11 20:11:02Z jari $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.hibernate;

import java.util.Collection;

import net.sf.basedb.core.UnsavedItemException;
import net.sf.basedb.core.data.IdentifiableData;

import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.type.Type;

/**
	Wraps a hibernate query and adds checks so that unsaved items aren't
	added to the query as parameters. This is normally not detected until the
	query is executed. By wrapping a query with this class the same situation is
	detected immediately when adding the parameter to the query, which makes
	it easier to find problems.
	
	@author nicklas
	@version 2.5
	@base.modified $Date: 2008-09-11 22:11:02 +0200 (Thu, 11 Sep 2008) $
*/
public class EntityQueryWrapper
	extends QueryWrapper
{
	public EntityQueryWrapper(Query query)
	{
		super(query);
	}
	
	@Override
	public Query setEntity(int position, Object val)
	{
		if (val instanceof IdentifiableData)
		{
			checkEntity((IdentifiableData)val);
		}
		return super.setEntity(position, val);
	}
	
	@Override
	public Query setEntity(String name, Object val)
	{
		if (val instanceof IdentifiableData)
		{
			checkEntity((IdentifiableData)val);
		}
		return super.setEntity(name, val);
	}
	
	@Override
	public Query setParameter(int position, Object val)
		throws HibernateException
	{
		if (val instanceof IdentifiableData)
		{
			checkEntity((IdentifiableData)val);
		}
		return super.setParameter(position, val);
	}
	
	@Override
	public Query setParameter(String name, Object val) 
		throws HibernateException
	{
		if (val instanceof IdentifiableData)
		{
			checkEntity((IdentifiableData)val);
		}
		return super.setParameter(name, val);
	}
	
	@Override
	public Query setParameter(int position, Object val, Type type)
	{
		if (val instanceof IdentifiableData)
		{
			checkEntity((IdentifiableData)val);
		}
		return super.setParameter(position, val, type);
	}
	
	@Override
	public Query setParameter(String name, Object val, Type type)
	{
		if (val instanceof IdentifiableData)
		{
			checkEntity((IdentifiableData)val);
		}
		return super.setParameter(name, val, type);
	}
	
	@Override
	public Query setParameterList(String name, Collection vals)
		throws HibernateException
	{
		checkEntityCollection(vals);
		return super.setParameterList(name, vals);
	}
	
	@Override
	public Query setParameterList(String name, Object[] vals)
		throws HibernateException
	{
		checkEntityArray(vals);
		return super.setParameterList(name, vals);
	}
	
	@Override
	public Query setParameterList(String name, Collection vals, Type type)
		throws HibernateException
	{
		if (type.isEntityType()) checkEntityCollection(vals);
		return super.setParameterList(name, vals, type);
	}
	
	@Override
	public Query setParameterList(String name, Object[] vals, Type type)
		throws HibernateException
	{
		if (type.isEntityType()) checkEntityArray(vals);
		return super.setParameterList(name, vals, type);
	}
	
	private void checkEntity(IdentifiableData entity)
	{
		if (entity.getId() == 0)
		{
			throw new UnsavedItemException("Can't use an unsaved item in query: " + entity);
		}
	}
	
	private void checkEntityCollection(Collection values)
	{
		for (Object val : values)
		{
			if (!(val instanceof IdentifiableData)) return;
			checkEntity((IdentifiableData)val);
		}
	}
	
	private void checkEntityArray(Object[] values)
	{
		for (Object val : values)
		{
			if (!(val instanceof IdentifiableData)) return;
			checkEntity((IdentifiableData)val);
		}
	}
}
