/*
	$Id: LogExpression.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;
import java.util.Collections;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.HibernateUtil;

/**
	Take the n-based logarithm of an expression: log(n, e).
	<p>
	The n-based logarithm is easily calculated as the natural logarithm of x
	divided natural logarithm of the base, ie. log(n, e) = ln(e) / ln(n)

	@author Nicklas
	@version 2.0
	@see Expressions#log(double, Expression)
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public class LogExpression
	implements Expression 
{
	private final double base;
	private final Expression e1;

	LogExpression(double base, Expression e1)
	{
		assert e1 != null : "e1 == null";
		assert base > 0.0 : "base <= 0";
		this.base = base;
		this.e1 = e1;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		// Get the log function that is returning the natural logarithm
		String argument = e1.toQl(query, dc);
		String ln = HibernateUtil.getDbEngine().ln(argument);
		if (HibernateUtil.getDbEngine().checkForInvalidNumberOperation())
		{
			ln = "(CASE WHEN " + argument + "<=0 THEN null ELSE " + ln + " END)";
		}
		if (base != Math.E)
		{
			double logBase = Math.log(base);
			ln = "(" + ln + " / " + logBase + ")";
		}
		return ln;
	}
	/**
		@return The expression
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Collections.singleton(e1);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		if (base == Math.E)
		{
			return "LN(" + e1 + ")";
		}
		else
		{
			return "LOG(" + base + ", " + e1 + ")";
		}
	}
	// -------------------------------------------
}

