/*
	$Id: JarClassLoader.java 5014 2009-06-25 13:11:37Z martin $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Enumeration;
import java.util.Vector;
import java.util.jar.JarFile;
import java.util.jar.JarEntry;
import java.util.jar.Manifest;
import java.util.jar.Attributes;
import java.io.File;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;

import net.sf.basedb.core.InvalidDataException;

/**
	A class loader implementation that loads classes from JAR files.
	Each JAR file requires a separate instance of this class. Use
	the {@link #getInstance(String)} method to get an existing or create
	a new instance for a specific JAR file. If the classes in the JAR file
	requires other classes in another JAR file to work, the paths to those
	JAR files must be listed in the <code>Class-Path</code> attribute in the
	<code>META-INF/MANIFEST.MF</code> file. For example:
	<pre class="code">
Manifest-Version: 1.0
Class-Path: OtherJarPlugin.jar
</pre> 

	If more than one JAR is needed separate them with one or more spaces.
	Note! It is only the <code>Class-Path</code> entry for the JAR file passed
	to the {@link #getInstance(String)} method that is checked. The manifest
	file is not checked for the other JARs.
	<p>
	This class loader will by default first look in the specified JAR files 
	for a class or resource and only if not found delegate to the parent class
	loader. This behaviour can be changed by calling {@link #setDelegateFirst(boolean)}
	or by setting <code>X-Delegate-First : true</code> in the <code>MANIFEST.MF</code>
	file in the main JAR file.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2009-06-25 15:11:37 +0200 (Thu, 25 Jun 2009) $
*/
public final class JarClassLoader
	extends ClassLoader
{

	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.util.JarClassLoader");
	
	/**
		A map of all loaded class loaders.
	*/
	private static final HashMap<String, JarClassLoader> classLoaders = 
		new HashMap<String, JarClassLoader>();

	/**
		Get a class loader for the specified JAR file. If a class loader for
		the specified file already exists, that class loader is returned, otherwise
		a new class loader is created.
		
		@param jarPath The path to a JAR file
		@return A class loader
		@throws IOException If the jar file can't be loaded
	*/
	public static final ClassLoader getInstance(String jarPath)
		throws IOException
	{
		return getInstance(jarPath, false);
	}

	/**
		Get a class loader for the specified JAR file, optionally unloading an the old
		one if the JAR file has been modified. A new class loader is created if no
		class loader exists or if <code>autoUnload</code> is <code>true</code> and the JAR file 
		has changed since the existing class loader was created. 
		
		@param jarPath The path to a JAR file
		@param autoUnload If TRUE the old class loaded will automatically be unloaded if the
			JAR file or any one it depends on (listed in the Class-Path attribute in the manifest file)
			has been modified (if the timestamp and/or size) is different
		@return A class loader
		@throws IOException If the jar file can't be loaded
		@since 2.4
	*/
	public static final ClassLoader getInstance(String jarPath, boolean autoUnload)
		throws IOException
	{
		JarClassLoader cl = classLoaders.get(jarPath);
		if (cl != null && autoUnload)
		{
			// Check if JAR file exists and has same timestamp and size
			if (cl.hasChanged(true)) cl = null;
		}
		if (cl == null)
		{
			cl = new JarClassLoader(jarPath);
			synchronized (classLoaders)
			{
				classLoaders.put(jarPath, cl);
			}
		}
		return cl;
	}
	
	
	/**
	 	Get a new class loader for the specified jar file.  
	  	@param jarPath The path to the jar file
	  	@return A class loader object
	  	@throws IOException If the jar file can't be loaded
	*/
	public static final ClassLoader newInstance(String jarPath)
		throws IOException
	{
		return new JarClassLoader(jarPath);
	}
	
	
	/**
		Unload the class loader for the given JAR file. The class loader will 
		not be unloaded if the calling application has object references to the 
		class loader or any class loaded by the class loader. However, if the
		{@link #getInstance(String)} method is called again, a new class loader
		instance will be created.
		
		@param jarPath The path to the JAR file
	*/
	public static final void unload(String jarPath)
	{
		synchronized (classLoaders)
		{
			classLoaders.remove(jarPath);
		}
		System.gc();
	}
	
	/**
		Check if a class loader for the given JAR file exists.
		@param jarPath The path to the JAR file
		@return TRUE if a class loader exists, FALSE otherwise
	*/
	public static final boolean exists(String jarPath)
	{
		return classLoaders.containsKey(jarPath);
	}
	
	/**
		The main JAR file to load classes from.
	*/
	private final File mainJarFile;

	/**
		Contains mappings from class names to the JAR file in which 
		the class implementation can be found.
	*/
	private final Map<String, List<File>> classPath;

	/**
		If we should delegate to the parent class loader first.
	*/
	private boolean delegateFirst;
	
	/**
		For keeping track of jar files listed by Class-Path entry
		in the manifest file.
	*/
	private final List<JarInfo> jarFiles;
	
	/**
		The timestamp of the JAR file.
	*/
	private final long jarTimeStamp;
	
	/**
		The size of the JAR file.
	*/
	private final long jarSize;
	
	
	/**
		Create a new JAR file class loader.
		@param jarPath The path to the JAR file
		@throws InvalidDataException If the JAR file can't be loaded
		@throws IOException If there is another IO-related error
	*/
	private JarClassLoader(String jarPath)
		throws IOException
	{
		super(Thread.currentThread().getContextClassLoader());
		log.debug("Creating JarClassLoader: file=" + jarPath);
		mainJarFile = new File(jarPath);
		if (!mainJarFile.exists() || !mainJarFile.isFile()) 
		{
			throw new InvalidDataException("JAR file not found: " + jarPath);
		}
		this.jarTimeStamp = mainJarFile.lastModified();
		this.jarSize = mainJarFile.length();
		this.delegateFirst = false;
		classPath = new HashMap<String, List<File>>();
		jarFiles = new ArrayList<JarInfo>();
		loadJarFile(mainJarFile, true);
	}

	/*
		From the ClassLoader class
		-------------------------------------------
	*/
	@Override
	protected Class<?> findClass(String name)
		throws ClassNotFoundException
	{
		log.debug("findClass: " + name);
		List<File> files = classPath.get(classNameToPath(name));
		if (files == null || files.isEmpty())
		{
			throw new ClassNotFoundException(name);
		}
		byte[] b = loadClassData(files.get(0), name);
		return defineClass(name, b, 0, b.length);
	}
	
	@Override
	protected URL findResource(String name)
	{
		log.debug("findResource: " + name);
		URL url = null;
		try
		{
			List<File> files = classPath.get(name);
			if (files != null && files.size() > 0)
			{
				if (log.isDebugEnabled()) log.debug("Found: " + files);
				String newUrl = "jar:"+files.get(0).toURI().toURL()+"!/"+name;
				url = new URL(newUrl);
			}
			else
			{
				log.debug("Not found: " + name);
			}
		}
		catch (MalformedURLException ex)
		{
			ex.printStackTrace();
		}
		return url;
	}
	
	@Override
	protected Enumeration<URL> findResources(String name)
	{
		log.debug("findResources: " + name);
		Enumeration<URL> url = null;
		try
		{
			List<File> files = classPath.get(name);
			if (files != null && files.size() > 0)
			{
				if (log.isDebugEnabled()) log.debug("Found: " + files);
				Vector<URL> v = new Vector<URL>(files.size());
				for (File f : files)
				{
					String newUrl = "jar:"+f.toURI().toURL()+"!/"+name;
					v.add(new URL(newUrl));
				}
				url = v.elements();
			}
			else
			{
				log.debug("Not found: " + name);
			}
		}
		catch (MalformedURLException ex)
		{
			ex.printStackTrace();
		}
		return url;
	}
	
	@Override
	protected Class<?> loadClass(String name, boolean resolve)
		throws ClassNotFoundException
	{
		if (log.isDebugEnabled()) log.debug("loadClass: " + name);
		
		// 1. Check if the class has already been loaded
		Class c = findLoadedClass(name);
		if (log.isDebugEnabled()) log.debug("loaded class (" + name + "): " + c);
		
		ClassLoader system = ClassLoader.getSystemClassLoader();
		ClassLoader parent = getParent();
		
		// 2. Check the system class loader
		if (c == null && parent != system) c = loadClassInternal(system, name);
		if (log.isDebugEnabled()) log.debug("system class (" + name + "): " + c);
		
		// 3. Delegate to parent class loader
		if (c == null && delegateFirst) c = loadClassInternal(parent, name);
		if (log.isDebugEnabled()) log.debug("parent class (" + name + "): " + c);

		// 4. Look for the class in this class path
		if (c == null)
		{
			try
			{
				c = findClass(name);
			}
			catch (ClassNotFoundException ex)
			{}
		}
		if (log.isDebugEnabled()) log.debug("my class (" + name + "): " + c);
		
		// 5. Delegate to parent class loader if it hasn't been done
		if (c == null && !delegateFirst)	c = loadClassInternal(parent, name);
		if (log.isDebugEnabled()) log.debug("parent class (" + name + "): " + c);
		
		if (c == null)
		{
			// Class not found
			throw new ClassNotFoundException(name);
		}
		
		if (resolve) resolveClass(c);
		return c;
	}
	
	@Override
	public URL getResource(String name)
	{
		if (log.isDebugEnabled()) log.debug("getResource: " + name);
		URL url = null;
		ClassLoader parent = getParent();
		
		// 1. Check with parent loader if delegating
		if (delegateFirst)
		{
			if (parent != null) url = parent.getResource(name);
		}
		
		// 2. Check with this class loader
		if (url == null) url = findResource(name);

		// 3. Check with parent loader if not done so already
		if (url == null && !delegateFirst)
		{
			if (parent != null) url = parent.getResource(name);
		}
		return url;
	}
	
	@Override
	public Enumeration<URL> getResources(String name)
		throws IOException
	{
		if (log.isDebugEnabled()) log.debug("getResources: " + name);
		ClassLoader parent = getParent();
		Vector<URL> resources = new Vector<URL>();
		
		Enumeration<URL> parentResources = null;
		if (parent != null) 
		{
			parentResources = parent.getResources(name);
		}
		Enumeration<URL> myResources = findResources(name);

		if (delegateFirst) addResources(resources, parentResources);
		addResources(resources, myResources);
		if (!delegateFirst) addResources(resources, parentResources);
		
		return resources.elements();
	}
	// -------------------------------------------

	/**
		If the class loader should delegate to the parent class loader
		before trying to find the class by it's own.
		@param delegateFirst TRUE to delegate to parent first, FALSE to
			only delegate if the class is not found
		@since 2.13
	*/
	public void setDelegateFirst(boolean delegateFirst)
	{
		this.delegateFirst = delegateFirst;
	}
	
	/**
		If this class loader delegates to the parent class loader
		before or after trying to find the class by itself.
		@since 2.13
	*/
	public boolean getDelegateFirst()
	{
		return delegateFirst;
	}
	
	/**
		Check if the JAR file this class loader loads is classes
		from has changed since this class loader was created. 
		@param checkSecondary TRUE to also check secondary JAR files
			listed in the Class-Path entry of the manifest file
		@since 2.8
	*/
	public boolean hasChanged(boolean checkSecondary)
	{
		boolean isSame = mainJarFile.exists() && 
			mainJarFile.lastModified() == jarTimeStamp && mainJarFile.length() == jarSize;
		if (!isSame) return true;
		if (checkSecondary)
		{
			for (JarInfo info : jarFiles)
			{
				File jarFile = info.jarFile;
				if (jarFile.exists())
				{
					isSame = info.existed && 
						jarFile.lastModified() == info.lastModified && 
						jarFile.length() == info.size;
				}
				else
				{
					isSame = !info.existed;
				}
				if (!isSame) return true;
			}
		}
		return false;
	}
	
	/**
		Convert a class name to a file path. Ie. replace dots with slahses and add .class
		to the end: net.sf.basedb.util.JarClassLoader -> net/sf/basedb/util/JarClassLoader.class
	*/
	private String classNameToPath(String className)
	{
		return className.replaceAll("\\.", "/")+".class";
	}
	
	/**
		Open the specified JAR file, list all entries and put them in the 
		{@link #classPath} mapping.
		@param file The JAR file to open
		@param followClassPath If the MANIFEST file should be checked for
			a <code>Class-Path</code> entry that lists other JAR files
			that also should be checked.
		@throws IOException If there is an error reading the JAR files
	*/
	private void loadJarFile(File file, boolean followClassPath)
		throws IOException
	{
		log.debug("loadJarFile: file=" + file);
		jarFiles.add(new JarInfo(file));
		if (!file.exists() || !file.isFile())
		{
			log.warn("File not found: " + file);
			return;
		}
		JarFile jarFile = new JarFile(file);
		Enumeration<JarEntry> entries = jarFile.entries();
		while (entries.hasMoreElements())
		{
			JarEntry jarEntry = entries.nextElement();
			if (jarEntry.isDirectory())
			{
				/* #### CONTINUE-STATEMENT #### */
				continue;
			}
			String name = jarEntry.getName();
			log.trace("Adding: " + name);
			List<File> list = classPath.get(name);
			if (list == null)
			{
				list = new ArrayList<File>();
				classPath.put(name, list);
			}
			list.add(file);
		}
		if (followClassPath)
		{
			Manifest manifest = jarFile.getManifest();
			if (manifest != null)
			{
				Attributes attr = manifest.getMainAttributes();
				if (attr != null)
				{
					String cp = attr.getValue(Attributes.Name.CLASS_PATH);
					log.debug("Class path of JAR file '" + file + "': " + cp);
					if (cp != null)
					{
						String[] cps = cp.split("\\s+");
						for (int i = 0; i < cps.length; ++i)
						{
							if (cps[i] != null && !cps[i].trim().equals(""))
							{
								loadJarFile(new File(file.getParent(), cps[i]), false);
							}
						}
					}
					if (file == mainJarFile)
					{
						delegateFirst = Values.getBoolean(attr.getValue("X-Delegate-First"), delegateFirst);
					}
				}
			}
		}
	}
	
	/**
		Load the byte[] of the given class. 
		
		@param file The JAR file in which the class implmentation is located
		@param name The name of the class using regular naming convention,
			ie. net.sf.basedb.util.JarClassLoader
		@throws ClassNotFoundException If the class can't be loaded
	*/
	private byte[] loadClassData(File file, String name)
		throws ClassNotFoundException
	{
		log.debug("loadClassData: " + name + " from file: " + file);
		InputStream in = null;
		try
		{
			JarFile jarFile = new JarFile(file);
			// Get the jar entry and open an input stream to read the file
			JarEntry jarEntry = jarFile.getJarEntry(classNameToPath(name));
			if (jarEntry == null)
			{
				throw new ClassNotFoundException(name+" in file "+file);
			}
			in = jarFile.getInputStream(jarEntry);
			int classSize = (int)jarEntry.getSize();
			byte[] b = new byte[classSize];
			int totalRead = 0;
			while (totalRead < classSize)
			{
				int numRead = in.read(b, totalRead, b.length - totalRead);
				if (numRead == -1) break;
				totalRead += numRead;
				log.debug("loadClassData: size=" + classSize + "; loaded=" + totalRead);
			}
			if (totalRead != classSize) 
			{
				throw new ClassFormatError("Could not read the complete class for " + 
					name + ": " + totalRead + " bytes; expected " + classSize + " bytes");
			}
			return b;
		}
		catch (Throwable ex)
		{
			throw new ClassNotFoundException(name, ex);
		}
		finally
		{
			// Close the input stream.
			try
			{
				if (in != null) in.close();
			}
			catch (IOException ex)
			{}
		}
	}
	
	private Class<?> loadClassInternal(ClassLoader loader, String name)
	{
		Class<?> c = null;
		if (loader != null)
		{
			try
			{
				c = loader.loadClass(name);
			}
			catch (ClassNotFoundException ex)
			{}
		}
		return c;
	}
	
	private void addResources(Vector<URL> resources, Enumeration<URL> e)
	{
		if (e == null) return;
		while (e.hasMoreElements())
		{
			resources.add(e.nextElement());
		}
	}
	
	static class JarInfo
	{
		final File jarFile;
		final boolean existed;
		final long lastModified;
		final long size;
		
		JarInfo(File jarFile)
		{
			this.jarFile = jarFile;
			this.existed = jarFile.exists();
			this.size = jarFile.length();
			this.lastModified = jarFile.lastModified();
		}
	}
}
