/**
	$Id: TableWriter.java 4974 2009-06-15 09:05:06Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.export;

import java.io.PrintWriter;
import java.io.Writer;

/**
	Wraps a writer stream and provides methods for easier writing
	of tabular data. The writer can be configured with a 
	{@link #setDataSeparator(String)} (default is 'tab') and
	{@link #setNullValue(String)} (default is an empty string).
	Use {@link #tablePrintData(Object...)} to print a singe line
	with data.

	@author Nicklas
	@version 2.12
	@base.modified $Date: 2009-06-15 11:05:06 +0200 (Mon, 15 Jun 2009) $
*/
public class TableWriter
	extends PrintWriter
{
	
	private String dataSeparator = "\t";
	private String nullValue = "";

	/**
		Create a new table writer that is writing it's output to the
		given writer.
		@param out The writer to write the data to
	*/
	public TableWriter(Writer out)
	{
		super(out);
	}

	/**
		The data separator string is written between
		each data column in the output. The default value is 'tab'.
		@return The current value of the data separator string
	*/
	public String getDataSeparator()
	{
		return dataSeparator;
	}
	
	/**
		Change the data separator string.
		@param dataSeparator The new data separator
	*/
	public void setDataSeparator(String dataSeparator)
	{
		this.dataSeparator = dataSeparator;
	}

	/**
		The null value string is written to the output instead of
		null data values. The default value is an empty string.
		@return The current value of the null value string
	*/
	public String getNullValue()
	{
		return nullValue;
	}
	
	/**
		Change the null value string.
		@param nullValue The new null value string
	*/
	public void setNullValue(String nullValue)
	{
		this.nullValue = nullValue;
	}
	
	/**
		Print a data line to the parent writer. Each value is separated by the
		data separator string ('tab' by default). Null values are replaced 
		with the null value string (empty string by default). Non-null objects
		are converted to strings values by calling their toString() method.

		@param data The values to write
	*/
	public void tablePrintData(Object... data)
	{
		boolean first = true;
		String ds = getDataSeparator();
		String nv = getNullValue();
		for (Object d : data)
		{
			if (!first) print(ds);
			print(d == null ? nv : d.toString());
			first = false;
		}
		println();
	}
	
}
