/**
	$Id: FieldConverter.java 4974 2009-06-15 09:05:06Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.export.spotdata;

import java.util.List;

import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ExtendedProperties;
import net.sf.basedb.core.ExtendedProperty;
import net.sf.basedb.core.ExtraValue;
import net.sf.basedb.core.ExtraValueType;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.IntensityTransform;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.RawDataProperty;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.ExtraValue.CoordinateType;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	Class for converting string-based spot and reporter names into 
	dynamic field representation suitable for bioassay set exporter.
	This class is for example used by the {@link net.sf.basedb.plugins.Base1PluginExecuter}
	to convert BASE 1 names into BASE 2 expressions. This feature rely
	in most cases on the use of formulas with the same name as the 
	BASE 1 field. BASE 1 fields that happens to have the same name in 
	BASE 2 doesn't need formulas.
	
	@author Nicklas
	@version 2.12
	@base.modified $Date: 2009-06-15 11:05:06 +0200 (Mon, 15 Jun 2009) $
*/
public class FieldConverter
{
	private final DbControl dc;
	private final BioAssaySet source;
	private ItemQuery<Formula> formulaQuery;
	private ItemQuery<ExtraValue> extraQuery;
	
	/**
		Create a new converter for the given bioassay set.
		@param dc A DbControl to use for database access
		@param source The source bioassay set
	*/
	public FieldConverter(DbControl dc, BioAssaySet source)
	{
		this.dc = dc;
		this.source = source;
	}
	
	/**
		Convert a reporter field. The conversion is done in steps
		until a match is found:
		<ul>
		<li>If the field starts with '_xc_' it could be an extra value. We
			check if there is a position extra value with an external id that is the
			same as the rest of the field (eg. without the _xc_ prefix).
		<li>Check if a formula that has the exactly the same name as the
			given field name exists. The formula must be a column expression
			and it must usable on the raw data type and the intensity transform 
			that the source bioassay set uses.
		<li>Check if an extended reporter property with exactly the same name
			as the given field name exists.
		</ul>
		@param fieldName The BASE 1 field name
		@return A DynamicField expression or null if the field name can't be 
			converted
	*/
	public DynamicField getReporterField(String fieldName)
	{
		DynamicField df = null;
		if (fieldName.startsWith("_xc_"))
		{
			df = getExtraValueField(fieldName.substring(4), CoordinateType.POSITION, fieldName);
		}
		if (df == null)
		{
			df = getFormulaField(fieldName, fieldName);
		}
		if (df == null)
		{
			df = getReporterProperty(fieldName, fieldName);
		}
		return df;
	}
	
	/**
		Convert a spot field. The conversion is done in steps
		until a match is found:
		<ul>
		<li>If the field starts with '_xc_' it could be an extra value. We
			check if there is an spot extra value with an external id that is the
			same as the rest of the field (eg. without the _xc_ prefix).
		<li>Check if a formula that has the exactly the same name as the
			given field name exists. The formula must be a column expression
			and it must usable on the raw data type and the intensity transform 
			that the source bioassay set uses.
		<li>Check if a raw data property with exactly the same name
			as the given field name exists.
		<li>Check if the field name is 'intensityX' where 'X' is a channel
			number.
		</ul>
		@param fieldName The BASE 1 field name
		@return A DynamicField expression or null if the field name can't be 
			converted
	*/
	public DynamicField getSpotField(String fieldName)
	{
		DynamicField df = null;
		if (fieldName.startsWith("_xc_"))
		{
			df = getExtraValueField(fieldName.substring(4), CoordinateType.SPOT, fieldName);
		}
		if (df == null)
		{
			df = getFormulaField(fieldName, fieldName);
		}
		if (df == null)
		{
			df = getRawdataProperty(fieldName, fieldName);
		}
		if (df == null)
		{
			df = getIntensityField(fieldName);
		}
		return df;
	}

	/**
		Find a formula with the given name and create a dynamic field from it
		using exportTitle as the title. The formula must be a 
		{@link Formula.Type#COLUMN_EXPRESSION} and be suitable for the 
		{@link RawDataType} and {@link IntensityTransform} used by the 
		source bioassay set.
		
		@param formulaName The name of the formula to look for
		@param exportTitle The exported title, or null to use the formula name
		@return A dynamic field, or null if no formula is found
	*/
	public DynamicField getFormulaField(String formulaName, String exportTitle)
	{
		if (formulaQuery == null)
		{
			formulaQuery = Formula.getQuery(Formula.Type.COLUMN_EXPRESSION, source.getRawDataType(), source.getIntensityTransform());
			formulaQuery.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
			formulaQuery.restrict(Restrictions.eq(Hql.property("name"), Expressions.parameter("name")));
		}
		formulaQuery.setParameter("name", formulaName, Type.STRING);
		List<Formula> result = formulaQuery.list(dc);
		DynamicField fld = null;
		if (result.size() > 0)
		{
			fld = ExportableFieldFactory.formula(result.get(0), exportTitle, null);
		}
		return fld;
	}
	
	/**
		Find an extra value with the given external id and coordinate type and
		create a dynamic field from it using exportTitle as the title. The extra
		value must of course be attached to the source bioassay set.
		
		@param externalId The external id of the {@link ExtraValueType}
		@param type The coordinate type of the extra value (eg. POSITION for reporter
			extra values, and SPOT for spot extra values)
		@param exportTitle The exported title or null to use the name (not the external id)
			of the extra value type
		@return A dynamic field, or null if no extra value is found
	*/
	public DynamicField getExtraValueField(String externalId, CoordinateType type, String exportTitle)
	{
		if (extraQuery == null)
		{
			extraQuery = source.getExtraValues();
			extraQuery.join(Hql.innerJoin("extraValueType", "evt"));
			extraQuery.restrict(Restrictions.eq(Hql.property("evt", "externalId"), 
					Expressions.parameter("externalId")));
			extraQuery.restrict(Restrictions.eq(Hql.property("dataCubeExtraValue.coordinateType"), 
					Expressions.parameter("type")));
		}
		extraQuery.setParameter("externalId", externalId, Type.STRING);
		extraQuery.setParameter("type", type.getValue(), Type.INT);
		List<ExtraValue> result = extraQuery.list(dc);
		DynamicField fld = null;
		if (result.size() > 0)
		{
			ExtraValueType evt = result.get(0).getExtraValueType();
			fld = ExportableFieldFactory.extraValue(evt, exportTitle, null);
		}
		return fld;
	}
	
	/**
		Find an extended reporter property with the given name and create
		a dynamic field from it using exportTitle as the title.
		@param name The name of the extended reporter property
		@param exportTitle The exported title or null to use the title
			of the extended property (which can be different from the name)
		@return A dynamic field, or null if no reporter property is found
	*/
	public DynamicField getReporterProperty(String name, String exportTitle)
	{
		DynamicField df = null;
		for (ExtendedProperty ep : ExtendedProperties.getProperties("ReporterData"))
		{
			if (ep.getName().equals(name))
			{
				if (exportTitle == null) exportTitle = ep.getTitle();
				df = ExportableFieldFactory.reporter(name, exportTitle, Formula.AverageMethod.MAX, null);
				break;
			}
		}
		return df;
	}
	
	/**
		Find an extended raw data property with the given name and create
		a dynamic field from it using exportTitle as the title.
		@param name The name of the extended raw data property
		@param exportTitle The exported title or null to use the title
			of the extended property (which can be different from the name)
		@return A dynamic field, or null if no raw data property is found
	*/
	public DynamicField getRawdataProperty(String name, String exportTitle)
	{
		DynamicField df = null;
		for (RawDataProperty rp : source.getRawDataType().getProperties())
		{
			if (rp.getName().equals(name))
			{
				df = ExportableFieldFactory.rawData(rp, exportTitle, null);
				break;
			}
		}
		return df;
	}
	
	/**
		Create a dynamic field for spot intensity if the field name has
		the pattern 'intensityX' where 'X' is a channel number between 1
		and the number of channels in the raw data type used by the 
		source bioassay set.
		@param fieldName The field name
		@return A dynamic field, or null if the name doesn't match the pattern
	*/
	public DynamicField getIntensityField(String fieldName)
	{
		DynamicField df = null;
		for (int i = 1; i <= source.getRawDataType().getChannels(); ++i)
		{
			if (fieldName.equals("intensity"+i))
			{
				df = ExportableFieldFactory.channel(i, "intensity"+i, null);
				break;
			}
		}
		return df;
	}

}
