/**
	$Id: JobAgentConnection.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.jobagent;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;

import net.sf.basedb.util.SocketUtil;

/**
	This class is used by client applications to communicate with job agents. 
	It used to send commands to job agents. For example, it can send
	<code>start</code>, <code>stop</code> and <code>info</code> commands.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (Thu, 11 Sep 2008) $
	@see JobAgentServerConnection
*/
public class JobAgentConnection
{

	private final String server;
	private final int port;
	private final int timeout;
	
	/**
		Create a connection to the local job agent running on the specified port.
		@param port The port the job agent is listening on
		@param timeout The timeout in milliseconds
	*/
	public JobAgentConnection(int port, int timeout)
	{
		this(null, port, timeout);
	}
	
	/**
		Create a connection to the job agent running on the specified server
		and listening on the specified port.
		@param server The server the job agent is running on, or null
			to connect to the local host
		@param port The port the job agent is listening on
		@param timeout The timeout in milliseconds
	*/
	public JobAgentConnection(String server, int port, int timeout)
	{
		this.server = server;
		this.port = port;
		this.timeout = timeout;
	}
	
	/**
		Send the <code>ping</code> command to the job agent. The only
		answer to this command is <code>OK</code>.
		@return OK if successful
		@throws IOException If there is an error
	 */
	public String sendPing()
		throws IOException
	{
		return send("ping");
	}

	/**
		Send a <code>start</code> request to the job agent.
		@return OK if successful
		@throws IOException If there is an error
	*/
	public String sendStart()
		throws IOException
	{
		return send("start");
	}
	
	public String sendStop()
		throws IOException
	{
		return send("stop");
	}
	
	public String sendPause()
		throws IOException
	{
		return send("pause");
	}
	
	/**
		Send an <code>info</code> or <code>status</code> request to the job agent.
		The difference is that a <code>status</code> request doesn't return
		information about cpu, memory or executing jobs.
		
		@param full If TRUE a <code>info</code> request is sent, othwerwise
			a <code>status</code> request is sent
		@return A <code>JobAgentInfo</code> object containing the 
			information
		@throws IOException If there is an error
	*/
	public JobAgentInfo getInfo(boolean full)
		throws IOException
	{
		return new JobAgentInfo(send(full ? "info" : "status"));
	}
	
	/**
		Close the connection to the job agent. In the current implementation
		all connections are stateless and this method does nothing. This might change
		in the future if a more advanced protocol is implemented. Clients should
		always call this method.
	*/
	public void close()
	{}
	
	/**
		Convenience method for opening a new socket, sending a command and
		return the answer.
		
		@param cmd The command to send
		@return The answer
		@throws IOException If there is an error
	 */
	private String send(String cmd)
		throws IOException
	{
		Socket socket = null;
		String answer;
		try
		{
			socket = connect();
			answer = sendCmd(socket, cmd);
		}
		finally
		{
			SocketUtil.close(socket);
		}
		return answer;
	}
	
	/**
		Create a socket connection to the job agent.
		@return The socket
		@throws IOException
	*/
	private Socket connect()
		throws IOException
	{
		Socket s = new Socket();
		InetSocketAddress address = null;
		if (server == null)
		{
			address = new InetSocketAddress(InetAddress.getLocalHost(), port);
		}
		else
		{
			address = new InetSocketAddress(server, port);
		}
		s.connect(address, timeout);
		return s;
	}

	/**
		Send a command to the job agent and return the answer. If the answer is
		<code>FAILED</code> an exception will be thrown.
		
		@param socket The socket used for communicating with the job agent
		@param cmd The command to send
		@return The answer from the job server
		@throws IOException If there is a communications error
	*/
	private String sendCmd(Socket socket, String cmd)
		throws IOException
	{
		// Send cmd to the job agent
		SocketUtil.send(socket, cmd, true);
		
		// Read the answer
		String answer = SocketUtil.read(socket, true);
		
		// The answer must start with OK, or it is an error
		if (answer == null || answer.length() == 0)
		{
			throw new IOException("No answer from "+socket.getInetAddress().toString());
		}
		else if (answer.indexOf("OK") != 0)
		{
			throw new IOException(answer.toString());
		}
		return answer.toString();
	}

}
