/*
	$Id: ColumnMapper.java 4515 2008-09-11 20:08:14Z jari $
	
	Copyright (C) 2006, 2007 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.
	
	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.parser;

import java.text.NumberFormat;
import java.text.ParsePosition;

import net.sf.basedb.core.Type;
import net.sf.basedb.util.parser.FlatFileParser.Data;

/**
	Returns the string found in the column given by the index
	set in the constructor. If the index is larger than the number
	of columns, an empty string is returned.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (Thu, 11 Sep 2008) $
*/
public class ColumnMapper
	implements Mapper
{
	private final int index;
	private final String name;
	private final NumberFormat parser;
	private final ParsePosition pos;
	private final boolean nullIfException;

	/**
		Create a new column mapper.
		@param index The index of the data column to use, starting at 0
		@param name An optional name of the column, use in the <code>toString</code>
			method
	 */
	public ColumnMapper(int index, String name)
	{
		this(index, name, null, false);
	}
	
	/**
		Create a new column mapper using a specific number format.
		@deprecated Use {@link ColumnMapper#ColumnMapper(int, String, NumberFormat, boolean)}
	*/
	public ColumnMapper(int index, String name, NumberFormat parser)
	{
		this(index, name, parser, false);
	}
	
	/**
		Create a new column mapper using a specific number format.
		@param index The index of the data column to use, starting at 0
		@param name An optional name of the column, use in the <code>toString</code>
			method
		@param nullIfException If TRUE, the mapper returns null for unparsable numeric
			values, otherwise an excption is thrown
		@param parser The parser to use or null to use Double.valueOf()
		@since 2.4
	*/
	public ColumnMapper(int index, String name, NumberFormat parser, boolean nullIfException)
	{
		this.index = index;
		this.name = name;
		this.parser = parser;
		this.pos = parser == null ? null : new ParsePosition(0);
		this.nullIfException = nullIfException;
	}
	
	/*
		From the Mapper interface
		-------------------------------------------
	*/
	/**
		Get the value in the data objects column with the index specified
		in the constructor.
		@return The value or null if the index is out of bounds
	*/
	public String getValue(Data data)
	{
		return index < data.columns() ? data.get(index) : null;
	}
	public Integer getInt(Data data)
	{
		Double d = getDouble(getValue(data));
		return d == null ? null : d.intValue();
	}
	public Float getFloat(Data data)
	{
		Double d = getDouble(getValue(data));
		return d == null ? null : d.floatValue();
	}
	// -------------------------------------------
	
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return name == null ? "\\" + index + "\\" : "\\" + name + "\\";
	}
	// -------------------------------------------
	
	private Double getDouble(String value)
	{
		if (value == null) return null;
		Double d = null;
		if (parser == null)
		{
			try
			{
				d = Double.valueOf(value);
			}
			catch (RuntimeException rex)
			{
				if (!nullIfException) throw rex;
			}
		}
		else
		{
			d = (Double)Type.DOUBLE.parseString(value, parser, nullIfException);
		}
		return d;
	}
	
}

