/**
	$Id: TestBioPlate.java 5060 2009-08-19 07:02:11Z nicklas $

	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioPlate;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.HardwareType;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemResultIterator;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.LabeledExtract;
import net.sf.basedb.core.MeasuredBioMaterial;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PlateGeometry;
import net.sf.basedb.core.Sample;
import net.sf.basedb.core.SystemItems;

import java.util.Date;

/**
	 @author martin
	 @version 2.10
	 @base.modified $Date: 2009-08-19 09:02:11 +0200 (Wed, 19 Aug 2009) $
 */

public class TestBioPlate 
{
	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}
	
	static boolean test_all()
	{
		write("++Testing bioplates");
		write_header();
		
		int sampleId = TestSample.test_create(0, "Sample A", true);
		int extractId = TestExtract.test_create(sampleId, true);
		int labelId = TestLabel.test_create(true);
		int labeledExtractId = TestLabeledExtract.test_create(extractId, labelId, true);
		int freezerId = TestHardware.test_create(SystemItems.getId(HardwareType.SCANNER), null, false);
		
		// Standard tests: create, load, list
		int plateGeometryId = TestPlateGeometry.test_create(8, 12, false);
		int id = test_create(plateGeometryId, freezerId, true);
		test_load(id);
		test_list(-1);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		// Standard test: Delete
		TestLabeledExtract.test_delete(labeledExtractId);
		TestLabel.test_delete(labelId);
		TestExtract.test_delete(extractId);
		TestSample.test_delete(sampleId);
		test_delete(id);
		TestPlateGeometry.test_delete(plateGeometryId);
		TestHardware.test_delete(freezerId);
		
		write("++Testing plates "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();		
			BioPlate bp = BioPlate.getById(dc, id);
			dc.deleteItem(bp);
			dc.commit();
			write("--Delete bioplate OK");
		}
		catch (Throwable ex)
		{
			write("--Delete bioplate FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<BioPlate> plates = BioPlate.getQuery().list(dc);
			for (int i = 0; i<plates.size(); i++)
			{
				write_item(i, plates.get(i));
			}
			if (expectedResults >=0 && expectedResults != plates.size())
			{
				throw new BaseException("Expected " + expectedResults + " results, " +
						"not " + plates.size());
			}
			write ("--List bioplates OK (" + plates.size() +")");
		}
		catch (Throwable ex)
		{
			write("--List bioplates FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioPlate bp = BioPlate.getById(dc, id);
			write("\tBio plate is used: " + bp.isUsed());
			write("--Load bioplate OK");
		}
		catch (Throwable ex)
		{
			write("--Load bioplate FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static int test_create(int plateGeometryId, int freezerId, boolean setAll)
	{
		if (plateGeometryId == 0 || !TestUtil.hasPermission(Permission.CREATE, Item.BIOPLATE))
		{
			return 0;
		}
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			PlateGeometry pg = PlateGeometry.getById(dc, plateGeometryId);			
			BioPlate bp = BioPlate.getNew(dc, pg);
			if (setAll)
			{
				bp.setName("Test BioPlate");
				bp.setDescription("Added at " + new Date());
				bp.setExternalId("test.bioplate.plate1");
				bp.setBarcode("995522");
				if (freezerId > 0) bp.setFreezer(Hardware.getById(dc, freezerId));
				ItemResultIterator<Sample> samples = Sample.getQuery().iterate(dc);
				ItemResultIterator<Extract> extracts = Extract.getQuery().iterate(dc);
				ItemResultIterator<LabeledExtract> labeledExtracts= LabeledExtract.getQuery().iterate(dc);
				int cols = pg.getColumns();
				int rows = pg.getRows();
				for (int row = 0; row < rows; row++)
				{
					for (int col = 0; col < cols; col++)
					{
						MeasuredBioMaterial mbm = null;
						if (samples.hasNext())
						{
							mbm = samples.next();
						}
						else if (extracts.hasNext())
						{
							mbm = extracts.next();
						}
						else if (labeledExtracts.hasNext())
						{
							mbm = labeledExtracts.next();
						}
						if (mbm != null) mbm.setBioWell(bp.getBioWell(row, col));
					}
				}
			}
			dc.saveItem(bp);
			dc.commit();
			id = bp.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(bp, false);
			write_item(0, bp);
			write("--Create bioplate OK");
		}
		catch (Throwable ex)
		{
			write("--Create bioplate FAIILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			System.out.println("   \tID \tExternal id\tName      \tDescription\tBarcode\tGeometry\tFreezer");
			System.out.println("-- \t-- \t-----------\t----------\t-----------\t-------\t---------\t--------");
		}
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void write_item(int i, BioPlate bioPlate)
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+bioPlate.getId()+"\t"+bioPlate.getExternalId()+"\t"+bioPlate.getName()+"\t"+bioPlate.getDescription()+
				"\t"+bioPlate.getBarcode()+"\t"+bioPlate.getPlateGeometry()+"\t"+bioPlate.getFreezer());
	}
}