import java.util.Date;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.ProtocolType;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.SystemItems;

/*
 $Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with BASE. If not, see <http://www.gnu.org/licenses/>.
 */

public class TestProtocol
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing protocol");
		write_header();
		// Standard tests: create, load, list
		int id1 = test_create(SystemItems.getId(ProtocolType.SAMPLING), null, true);
		int id2 = test_create(SystemItems.getId(ProtocolType.LABELING), null, false);
		test_load(id1);
		test_list();

		// Extra tests: attach file to protocol
		int fileId = TestFile.test_create("test.upload.txt", false, false);
		test_attachfile(id1, fileId);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_deattachfile(id1);
		TestFile.test_delete(fileId);

		// Standard test: Delete
		test_delete(id1);
		test_delete(id2);
		write("++Testing protocol "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	static int test_create(int protocoltypeId, String name, boolean setAll)
	{
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ProtocolType pt = ProtocolType.getById(dc, protocoltypeId);
			Protocol p = Protocol.getNew(dc, pt);
			if(setAll)
			{
				p.setName("Test protocol");
				p.setDescription("Added at "+new Date());
			}
			if (name != null) p.setName(name);
			dc.saveItem(p);
			dc.commit();
			id = p.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(p, false);
			write_item(0, p);
			write("--Create protocol OK");
		}
		catch (Throwable ex)
		{
			write("--Create protocol FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Protocol p = Protocol.getById(dc, id);
			dc.deleteItem(p);
			dc.commit();
			write("--Delete protocol OK");
		}
		catch (Throwable ex)
		{
			write("--Delete protocol FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Protocol p = Protocol.getById(dc, id);
			write_item(0, p);
			write("--Load protocol OK");
		}
		catch (Throwable ex)
		{
			write("--Load protocol FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Protocol> l = Protocol.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List protocols OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List protocols FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tType\tFile");
			write("-- \t-- \t--------- \t-----------\t----\t----");
		}
	}
	static void write_item(int i, Protocol p)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
			write(i+":\t"+p.getId()+"\t"+p.getName()+"\t"+p.getDescription()+"\t"+p.getProtocolType().getName()+"\t"+p.getFile());
	}
	static void write(String message)
	{
		System.out.println(message);
	}
	
	private static void test_deattachfile(int protocolId)
	{
		if (protocolId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Protocol p = Protocol.getById(dc, protocolId);
			p.setFile(null);
			dc.commit();
			dc = TestUtil.getDbControl();
			dc.reattachItem(p, false);
			write_item(0, p);
			write("--Deattach file OK");
		}
		catch (Throwable ex)
		{
			write("--Deattach file FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	private static void test_attachfile(int protocolId, int fileId)
	{
		if (fileId == 0 || protocolId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Protocol p = Protocol.getById(dc, protocolId);
			File f = File.getById(dc, fileId);
			f.setName("attachfile");
			p.setFile(f);
			dc.commit();
			dc = TestUtil.getDbControl();
			dc.reattachItem(p, false);
			write_item(0, p);
			write("--Attach file OK");
		}
		catch (Throwable ex)
		{
			write("--Attach file FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_set_protocol_parameter(int protocolId, int annotationTypeId)
	{
		if (annotationTypeId == 0 || protocolId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Protocol p = Protocol.getById(dc, protocolId);
			AnnotationType at = AnnotationType.getById(dc, annotationTypeId);
			at.setProtocolParameter(true);
			p.addParameter(at);
			dc.commit();
			write("--Set protocol parameter OK");
		}
		catch (Throwable ex)
		{
			write("--Set protocol parameter FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

}
