/**
	$Id: BioAssaySetClient.java 4931 2009-05-14 11:16:12Z nicklas $

	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.client;

import java.io.IOException;
import java.io.InputStream;

import net.sf.basedb.info.DataFileTypeInfo;
import net.sf.basedb.info.QueryOptions;

import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.client.Options;
import org.apache.axis2.rpc.client.RPCServiceClient;

/**
 	Web service client for BioAssaySetService in BASE.
 	{@link net.sf.basedb.ws.server.BioAssaySetService}
  	@author Martin
  	@version 2.5
 */
public class BioAssaySetClient 
	extends AbstractRPCClient
{
	private final SessionClient session;
	
	/**
	 	Creates new BioAssayClient for a session.
	  	@param session Active session.
	 */
	public BioAssaySetClient(SessionClient session)
	{
		super(session.getUrl(), "BioAssaySet", session.getServiceFactory());
		this.session = session;
	}
	
	/**
	 	Calls the BioAssaySetService and gets the available formats to export
	 	a BioAssaySet from BASE in.
	  	@return An array with the formats. 
	  	@throws AxisFault If communication with the service fails.
	  	@deprecated In 2.12. Use {@link #getDataFileTypes(int, QueryOptions)} instead.
	*/
	public String[] getExportFormats()
		throws AxisFault
	{
		String[] formats = invokeBlocking("getExportFormats", String[].class, session.getId());
		return formats;
	}
	
	/**
	 	Call the service in BioAssaySetService that exports a bioassay set and
	 	then sends it as a file. 
	  	@param bioAssaySetId Id of the bioAssay set to export
	  	@param format Export format to use when exporting the bioassay set
	  	@param reporterFields Reporter fields to include in the export
	  	@param spotFields Spot fields to include in the export
	  	@param mergeReporters If the reporters should be merged or not.
	  	@return An InputStream to read the file data from
	  	@throws AxisFault If communication with services fails.
	  	@throws IOException If accessing the file files.
	  	@deprecated In 2.12. 
	  		See {@link net.sf.basedb.ws.server.BioAssaySetService#downloadBioAssaySet(String, int, String, String[], String[], Boolean)}
	  		for more information. Use {@link #downloadDataFile(int, String)} instead.
	*/
	public InputStream downloadBioAssaySet(int bioAssaySetId, String format, 
			String[] reporterFields, String[] spotFields, boolean mergeReporters) 
		throws AxisFault, IOException
	{
		RPCServiceClient service = getService();
		Options options = service.getOptions();		
		options.setProperty(Constants.Configuration.ENABLE_MTOM, Constants.VALUE_TRUE);
		service.setOptions(options);
		return invokeFileBlocking
			(
				"downloadBioAssaySet",
				session.getId(), 
				bioAssaySetId, 
				format,
				reporterFields,
				spotFields,
				mergeReporters
			);			
	}
	
	/**
	 	Returns if a bioassay set has spot data in the form of files that 
	 	can be downloaded or not.
	  	@param bioAssaySetId Id of the bioassay set it concerns.
	  	@return TRUE if there are data to be downloaded, FALSE otherwise.
	  	@throws AxisFault If the communication with the webservice fails.
	  	@since 2.12
	 */
	public Boolean hasDownloadableData(int bioAssaySetId) 
		throws AxisFault
	{
		return invokeBlocking
			(
				"hasDownloadableData",
				Boolean.class,
				session.getId(),
				bioAssaySetId
			);
	}

	/**
	 	Returns information about data file types that are used storing
	 	spot data for a bioassay set.
	  	@param bioAssaySetId Id of the bioassay set to get the information about.
	  	@param qOpt Restrictions and includes to put on the query when getting the information.
	  	@return An array of DataFileTypeInfo, one for each data file type used in the raw bioassay.
	  	@throws AxisFault If communication with webservice fails.
	  	@since 2.12
	*/
	public DataFileTypeInfo[] getDataFileTypes(int bioAssaySetId, QueryOptions qOpt)
		throws AxisFault
	{
		return invokeBlocking
			(
				"getDataFileTypes",
				DataFileTypeInfo[].class,
				session.getId(),
				bioAssaySetId,
				qOpt
			);
	}

	/**
	 	Gets spot data as a file for a bioassay set if it is stored in a file
	 	in BASE.
	  	@param bioAssaySetId Id of bioassay set to get the file from
	  	@param dataFileType The external ID of the data file type to download
	  	@return An InputStream to read the file data from, or null
	  		if no file of the specified type exists
	  	@throws AxisFault If communication with webservices fails.
	  	@throws IOException If writing to the target file fails.
	  	@since 2.12
	*/
	public InputStream downloadDataFile(int bioAssaySetId, String dataFileType) 
		throws AxisFault, IOException
	{
		RPCServiceClient service = getService();
		Options options = service.getOptions();		
		options.setProperty(Constants.Configuration.ENABLE_MTOM, Constants.VALUE_TRUE);
		service.setOptions(options);
		return invokeFileBlocking
			(
				"downloadDataFile", 
				session.getId(), 
				bioAssaySetId,
				dataFileType
			);
	}

	
}
